

import xmlrpclib
import ValueTree


class RpcError:
    def __init__(self, code, string):
        self.errorCode = code
        self.errorString = string

class DeviceException:
    def __init__(self, errors):
        self.errorList = errors

class Device:
    def __init__(self, url, rootParameter):
        self.server = xmlrpclib.ServerProxy(url)
        self.rootParameter = rootParameter

        try:            
            deviceState = self.server.get(rootParameter)
            self.values = ValueTree.ValueTree(rootParameter, deviceState)
        except xmlrpclib.Fault, fault:
            self._FaultHandler(fault)

    def _FaultHandler(self, fault):
        # we'll convert the fault message into 
        errorList = []
        if fault.faultCode == "RpcErrors":
            # there were multiple errors -- get them all...
            errors = self.server.getErrors()
            for error in errors:
                thisError = RpcError(error["errorCode"], error["errorString"])
                errorList.append(thisError)
        else:
            errorList.append(RpcError(fault.faultCode, fault.faultString))

        raise DeviceException(errorList)


    def Query(self, parameter, index=None):
        try:
            if index is not None:
                newValue = self.server.get(parameter, index)
            else:
                newValue = self.server.get(parameter)
                
            self.values.SetValue(parameter, newValue, index)
            return newValue

        except xmlrpclib.Fault, fault:
            self._FaultHandler(fault)

    def GetValue(self, parameter, index=None):
        return self.values.GetValue(parameter, index)

    def SetValue(self, parameter, value, index=None):
        try:
            if index is not None:
                status = self.server.set(parameter, index, value)
            else:
                status = self.server.set(parameter, value)

            self.values.SetValue(parameter, value, index)

        except xmlrpclib.Fault, fault:
            self._FaultHandler(fault)

          
    




if __name__ == "__main__":
    d = Device("http://localhost:8080/goform/RPC2", "system")
    print d.Query("system.info.uptime")

    d.SetValue("system.info.name", "New Name!")
    print d.GetValue("system.info.name")
    
    try:
        d.SetValue("zzz.xxx.ccc", 10101, 4)
    except DeviceException, e:
        for error in e.errorList:
            print "ERROR Code = %s\n      String = %s" % (error.errorCode, \
             error.errorString)


        

    
