

from Tkinter import *
import Device



kNumber = 0
kLabel = 1
kLowerAlarm = 2
kUpperAlarm = 3
kValue = 4



class RpcEntry(Entry):
    """
        Helpful utility class that extends the Tkinter 'Entry' widget.
        We store the parameter name and index that this entry widget is
        responsible for.
    """
    def __init__(self, name=None, index = None, parent=None, **config):
        Entry.__init__(self, parent, config)
        self.name = name
        self.index = index
        self.focus = 0

    def Focus(self):
        self.focus = 1

    def Blur(self):
        self.focus = 0

    def UpdateServer(self, device, value):
        if device:
            device.SetValue(self.name, value, self.index)
        
        
class RpcLabel(Label):
    """ Stub class so we can create either a readonly label or an editable
        entry box using the same interface.
    """
    def __init__(self, name=None, index = None, parent=None, **config):
        Label.__init__(self, parent, config)
        self.name = name
        self.index = index
        self.focus = 0

    def UpdateServer(self, app, value):
        pass

class Row(Frame):
    def __init__(self, index = None, device = None, parent=None, **config):
        Frame.__init__(self, parent, config)
        self.pack()
        self.device = device
        self.index = index
        self.lowerAlarm = 0
        self.upperAlarm = 100
        self.lastFocus = ""
        self.Build()

        
    def Build(self):
        self.number = Label(self, borderwidth=1, relief=GROOVE, anchor=W, width=3)
        self.number.grid(row=0, column=kNumber)

        labelType = RpcEntry
        if self.index is None:
            labelType = RpcLabel

        self.labelText = StringVar()
        self.label = labelType("system.values.label", self.index, self, \
                               textvariable=self.labelText, borderwidth=1, relief=GROOVE,  width=32)
        
        self.label.grid(row=0, column=kLabel)
        self.label.bind("<FocusIn>", self.Focus)
        self.label.bind("<FocusOut>", self.Blur)


        self.lowerText = StringVar()
        self.lower = labelType("system.values.lowerAlarm", self.index, self, \
                               textvariable=self.lowerText, borderwidth=1, relief=GROOVE, width=15)
        self.lower.grid(row=0, column = kLowerAlarm)
        self.lower.bind("<FocusIn>", self.Focus)
        self.lower.bind("<FocusOut>", self.Blur)

        self.upperText = StringVar()        
        self.upper = labelType("system.values.upperAlarm", self.index, self, \
                               textvariable=self.upperText, borderwidth=1, relief=GROOVE, width=15)
        self.upper.grid(row=0, column = kUpperAlarm)
        self.upper.bind("<FocusIn>", self.Focus)
        self.upper.bind("<FocusOut>", self.Blur)



        self.value = RpcLabel("system.values.value", self.index, self, \
                              borderwidth=1, relief=GROOVE, width=8)
        self.value.grid(row=0, column = kValue)

    def Focus(self, event):
        event.widget.Focus()
        self.lastFocus = event.widget.get()

        

    def Blur(self, event):
        if self.lastFocus != event.widget.get():
            event.widget.UpdateServer(self.device, event.widget.get())
        event.widget.Blur()
        

    def SetDevice(self, device):
        self.device = device

    def Update(self):
        # update all our values from the Device object...
        device = self.device
        val = device.GetValue(self.label.name, self.index)
        self.SetLabel(val)

        val = device.GetValue(self.lower.name, self.index)
        self.SetLowerAlarm(val)

        val = device.GetValue(self.upper.name, self.index)
        self.SetUpperAlarm(val)

        val = device.GetValue(self.value.name, self.index)
        self.SetValue(val)

    def SetNumber(self, num):
        self.number["text"] = str(num)
        
    def SetLabel(self, label):
        if not self.label.focus:
            self.labelText.set(label)
        

    def SetLowerAlarm(self, val):
        if not self.lower.focus:
            self.lowerText.set(val)
            try:
                self.lowerAlarm = int(val)
            except:
                pass

    def SetUpperAlarm(self, val):
        if not self.upper.focus:
            self.upperText.set(val)
            try:
                self.upperAlarm = int(val)
            except:
                pass

    def SetValue(self, val):
        try:
            intVal = int(val)
            bgColor = "Gray"
            fgColor = "Black"
            if intVal <= self.lowerAlarm:
                bgColor = "Blue"
                fgColor = "Yellow"
            elif intVal >= self.upperAlarm:
                bgColor = "Red"
                fgColor = "White"
                
            #self.value["background"] = widget.winfo_rgb(bgColor)
            self.value["background"] = bgColor
            self.value["foreground"] = fgColor
        except:
            pass
        self.value["text"] = str(val)
        
        

if __name__ == "__main__":
    r = Row(0)
    r.SetNumber(1)
    r.SetLabel("This is a row!")
    r.SetLowerAlarm(30)
    r.SetUpperAlarm(50)
    r.SetValue(20)
    r.mainloop()
    
