/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *
 * The code in this file was originally derived from the 
 * Java source code of The Jakarta Project (Tomcat), 
 * which is copyrighted by The Apache Software Foundation (Their 
 * copyright notice shown below). 
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * The original code may have completely changed while converting
 * to C++. It should be noted that only the packages that contain 
 * the Java Servlet API interfaces, namely javax.servlet and 
 * javax.servlet.http of the Jakarta Project are used for 
 * implementing C++WSP. The rest of the code including all the 
 * modifications to the files derived from these two packages is 
 * copyrighted by Infosys Technologies Ltd.
 */

#ifndef COOKIE_H
#define COOKIE_H

#include <string>
#include "base_servlet/ServletException.h"


// If you want to support new cookies 
// comment out this line
#define OLD_COOKIE 1





class Cookie  {



    
 private:
	 std::string _name;	
     std::string _value;	

   
     std::string _comment;	
				
     std::string _domain;	
     int _maxAge;	
     std::string _path;	
     bool _secure;	
     int _version;	
    
    


 public:

	 // Adding a constructor which parses a header value to 
	 // initialize a "Cookie" header value . This is only for
	 // requests so it won't work for parsng "Set-Cookie" 
	 // values

	 Cookie():_maxAge(-1),
	 _version(1), _secure(false)
	 {}

	 // defaulting the cookie version to 1
	 // as if no version is provided the default behaviour is 
	 // to ignore the Max-Age and a host of other parameters set 
	 // on the cookie
	 Cookie(std::string name, std::string value):_maxAge(-1),
	 _version(1), _secure(false) {
	if (   name== "Comment"	
		|| name == "Discard"
		|| name == "Domain"
		|| name == "Expires"	
		|| name == "Max-Age"	
		|| name == "Path"
		|| name == "Secure"
		|| name == "Version"
	    ) {
	    std::string errMsg = "cookie name is token";
	    throw new ServletException(errMsg);
	}

	this->_name = name;
	this->_value = value;
    }





     void setComment(std::string purpose) {
	_comment = purpose;
    }
    
    
    

     std::string getComment() {
	return _comment;
    }
    


     void setDomain(std::string pattern) {
		 _domain = pattern;
    }
    
    

     std::string getDomain() {
	return _domain;
    }


     void setMaxAge(int expiry) {
	_maxAge = expiry;
    }


     int getMaxAge() {
	return _maxAge;
    }
    
    

     void setPath(std::string uri) {
	_path = uri;
    }

     std::string getPath() {
	return _path;
    }


 
     void setSecure(bool flag) {
	_secure = flag;
    }


     bool getSecure() {
	return _secure;
    }


	 void setName(const char *nm)
	 { _name = nm; }


     std::string getName() {
	return _name;
    }





     void setValue(std::string newValue) {
	_value = newValue;
    }





     std::string getValue() {
	return _value;
    }





     int getVersion() {
	return _version;
    }




     void setVersion(int v) {
	_version = v;
    }

};

#endif // COOKIE_H