/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *
 * The code in this file was originally derived from the 
 * Java source code of The Jakarta Project (Tomcat), 
 * which is copyrighted by The Apache Software Foundation (Their 
 * copyright notice shown below). 
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * The original code may have completely changed while converting
 * to C++. It should be noted that only the packages that contain 
 * the Java Servlet API interfaces, namely javax.servlet and 
 * javax.servlet.http of the Jakarta Project are used for 
 * implementing C++WSP. The rest of the code including all the 
 * modifications to the files derived from these two packages is 
 * copyrighted by Infosys Technologies Ltd.  
 * 
 */

#ifndef HTTPSERVLET_H
#define HTTPSERVLET_H



#include <string>

#include "base_servlet/GenericServlet.h"
#include "HttpServletRequest.h"
#include "HttpServletResponse.h"

#include "cpp_servlet_dll.h"


class NoBodyOutputStream : public  ServletOutputStream {

private:

     int contentLength;

public:

    NoBodyOutputStream() {}

    int getContentLength() {
	return contentLength;
    }

     void write(int b) {
	contentLength++;
    }

	 // Check this method - providing only an empty 
	 // implemnetation for now - K Naqvi
	 void flush()
	 {}

     void write(char buf[], int offset, int len)
	throw (ServletException)
    {
	if (len >= 0) {
	    contentLength += len;
	} else {
	    
		std::string msg = "err.io.negativelength";
	    throw new ServletException("negative length");
	}
    }
};


class NoBodyResponse : public  HttpServletResponse {
private:
	HttpServletResponse		*resp;
     NoBodyOutputStream		noBody;
     bool			didSetContentLength;

public:

    NoBodyResponse(HttpServletResponse *r):didSetContentLength(false) {
	resp = r;
    }

	void sendHeaders() throw(ServletException)
	{}

    void setContentLength() {
	if (!didSetContentLength)
	  resp->setContentLength(noBody.getContentLength());
    }



     void setContentLength(int len) {
	resp->setContentLength(len);
	didSetContentLength = true;
    }

     void setContentType(std::string type)
      { resp->setContentType(type); }

     ServletOutputStream & getOutputStream() throw( ServletException)
      { return noBody; }

     std::string getCharacterEncoding()
	{ return resp->getCharacterEncoding(); }


     void setBufferSize(int size) throw (ServletException)
      { resp->setBufferSize(size); }

     int getBufferSize()
      { return resp->getBufferSize(); }

     void reset() throw (ServletException)
      { resp->reset(); }
      
       void resetBuffer() throw (ServletException)
      { resp->resetBuffer(); }

     bool isCommitted()
      { return resp->isCommitted(); }

     void flushBuffer() throw (ServletException)
      { resp->flushBuffer(); }

     void setLocale(std::string loc)
      { resp->setLocale(loc); }

	 std::string getLocale()
      { return resp->getLocale(); }



     void addCookie(Cookie cookie)
      { resp->addCookie(cookie); }

     bool containsHeader(std::string name)
      { return resp->containsHeader(name); }


     void setStatus(int sc)
      { resp->setStatus(sc); }

     void setHeader(std::string name, std::string value)
      { resp->setHeader(name, value); }

     void setIntHeader(std::string name, int value)
      { resp->setIntHeader(name, value); }

     void setDateHeader(std::string name, long date)
      { resp->setDateHeader(name, date); }

     void sendError(int sc, std::string msg) throw (ServletException)
      { resp->sendError(sc, msg); }

     void sendError(int sc) throw (ServletException)
      { resp->sendError(sc); }

     void sendRedirect(std::string location) throw (ServletException)
      { resp->sendRedirect(location); }
    
     std::string encodeURL(std::string url) 
      { return resp->encodeURL(url); }

     std::string encodeRedirectURL(std::string url)
      { return resp->encodeRedirectURL(url); }
      
     void addHeader(std::string name, std::string value)
      { resp->addHeader(name, value); }
      
     void addDateHeader(std::string name, long value)
      { resp->addDateHeader(name, value); }
      
     void addIntHeader(std::string name, int value)
      { resp->addIntHeader(name, value); }
      
      
      

     
     
     std::string encodeUrl(std::string url) 
      { return this->encodeURL(url); }
      
      
      
      
      
      
      
    
     
     std::string encodeRedirectUrl(std::string url)
      { return this->encodeRedirectURL(url); }

};








class CPP_SERVLET_DLL_API HttpServlet : public  GenericServlet
{
private:
	static  std::string METHOD_DELETE;
     static  std::string METHOD_HEAD;
     static  std::string METHOD_GET;
     static  std::string METHOD_OPTIONS;
     static  std::string METHOD_POST;
     static  std::string METHOD_PUT;
     static  std::string METHOD_TRACE;
     static  std::string HEADER_IFMODSINCE;
     static  std::string HEADER_LASTMOD;
    
   
   
	public:  
    
    /**
     * Does nothing, because this is an abstract class.
     * 
     */


	HttpServlet() { }

	virtual ~HttpServlet() {}

	/** 
	init is not needed here - GenericServlet definition should
	be good enough.
	 virtual void init(ServletConfig &config) throw( ServletException)
	 {
		 GenericServlet::init(config);
	 }
	 **/

	 // Providing empty implementation for destroy
	 virtual void destroy()
	 {}

    
    protected:



	virtual void doGet(HttpServletRequest & req, HttpServletResponse & resp)
	throw( ServletException)
    {
	std::string protocol = req.getProtocol();
	std::string msg = "http.method_get_not_supported";
	if (std::string::npos != protocol.find("1.1")) {
	    resp.sendError(HttpServletResponse.SC_METHOD_NOT_ALLOWED, msg);
	} else {
	    resp.sendError(HttpServletResponse.SC_BAD_REQUEST, msg);
	}
    }





     long getLastModified(HttpServletRequest & req) {
	return -1;
    }



     virtual void doHead(HttpServletRequest & req, HttpServletResponse & resp)
	throw (ServletException)
    {
	NoBodyResponse response(&resp);
	
	doGet(req, response);
	response.setContentLength();
    }
    

     virtual void doPost(HttpServletRequest & req, HttpServletResponse & resp)
	throw (ServletException)
    {
	std::string protocol = req.getProtocol();
	std::string msg = "http.method_post_not_supported";
	if (std::string::npos != protocol.find("1.1")) {
	    resp.sendError(HttpServletResponse.SC_METHOD_NOT_ALLOWED, msg);
	} else {
	    resp.sendError(HttpServletResponse.SC_BAD_REQUEST, msg);
	}
    }


  
     virtual void doPut(HttpServletRequest & req, HttpServletResponse & resp)
	throw (ServletException)
    {
	std::string protocol = req.getProtocol();
	std::string msg = "http.method_put_not_supported";
	if (std::string::npos != protocol.find("1.1")) {
	    resp.sendError(HttpServletResponse.SC_METHOD_NOT_ALLOWED, msg);
	} else {
	    resp.sendError(HttpServletResponse.SC_BAD_REQUEST, msg);
	}
    }

     
     virtual void doDelete(HttpServletRequest & req,
			    HttpServletResponse & resp)
	throw (ServletException)
    {
	std::string protocol = req.getProtocol();
	std::string msg = "http.method_delete_not_supported";
	if (std::string::npos != protocol.find("1.1")) {
	    resp.sendError(HttpServletResponse.SC_METHOD_NOT_ALLOWED, msg);
	} else {
	    resp.sendError(HttpServletResponse.SC_BAD_REQUEST, msg);
	}
    }
    




    private:


	protected:

         
    
		void doOptions(HttpServletRequest & req, HttpServletResponse & resp)
	throw (ServletException)
    {
    }
    
    
    
    

     void doTrace(HttpServletRequest & req, HttpServletResponse & resp) 
	throw (ServletException)
    {
	
    }		





     void service_http(HttpServletRequest * req, HttpServletResponse * resp)
	throw (ServletException)
    {
	   
	std::string method = req->getMethod();

	if (method.compare(METHOD_GET) == 0) {
	    long lastModified = getLastModified(*req);
	    if (lastModified == -1) {
		// servlet doesn't support if-modified-since, no reason
		// to go through further expensive logic
		doGet(*req, *resp);
	    } else {
		long ifModifiedSince = req->getDateHeader(HEADER_IFMODSINCE);
		if (ifModifiedSince < (lastModified / 1000 * 1000)) {
		    // If the servlet mod time is later, call doGet()
                    // Round down to the nearest second for a proper compare
                    // A ifModifiedSince of -1 will always be less
		    maybeSetLastModified(*resp, lastModified);
		    doGet(*req, *resp);
		} else {
		    resp->setStatus(HttpServletResponse.SC_NOT_MODIFIED);
		}
	    }

	} else if (0 == method.compare(METHOD_HEAD)) {
	    long lastModified = getLastModified(*req);
	    maybeSetLastModified(*resp, lastModified);
	    doHead(*req, *resp);

	} else if (0 == method.compare(METHOD_POST)) {
	    doPost(*req, *resp);
	    
	} else if (0 == method.compare(METHOD_PUT)) {
	    doPut(*req, *resp);	
	    
	} else if (0 == method.compare(METHOD_DELETE)) {
	    doDelete(*req, *resp);
	    
	} else if (0 == method.compare(METHOD_OPTIONS)) {
		doOptions(*req,*resp);
	    
	} else if (0 == method.compare(METHOD_TRACE)) {
	    doTrace(*req,*resp);
	    
	} else {
	    //
	    // Note that this means NO servlet supports whatever
	    // method was requested, anywhere on this server.
	    //

	    std::string errMsg = "http.method_not_implemented";
	    
	    resp->sendError(HttpServletResponse.SC_NOT_IMPLEMENTED, errMsg);
	}
    }
    




    private:
		void maybeSetLastModified(HttpServletResponse &resp,
				      long lastModified) {
	if (resp.containsHeader(HEADER_LASTMOD))
	    return;
	if (lastModified >= 0)
	    resp.setDateHeader(HEADER_LASTMOD, lastModified);
    }
   
   
   public:
    

	
      void service(ServletRequest & req, ServletResponse & res)
	throw (ServletException)
    {
		try 
		{
			//HttpServletRequest	& request = (HttpServletRequest &) req;
			//HttpServletResponse	& response = (HttpServletResponse &) res;
			service_http((HttpServletRequest *)&req, (HttpServletResponse *)&res);
		} 
		catch (...) 
		{
			throw new ServletException("non-HTTP request or response");
		}
    }
};






#endif // HTTPSERVLET_H
