/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#pragma warning ( disable : 4786 )

#include <time.h>
#include <ace/OS.h>
#include <ace/OS_String.h>
#include "HttpServletRequest.h"

HttpServletRequest::HttpServletRequest():_httpSessionObj(NULL),
_newSessionCreated(false), _oldSessionValid(false)
{
}

void HttpServletRequest::addHeader(const char * hdrKey, const char *hdrVal)
{
	// search till the first space and then store the key and value to
	// to _headerList
	//std::string keyValStr(hdr);

	//int colPos = keyValStr.find_first_of(':');
	//if( std::string::npos == colPos)
	//	return; // Illegal header

	std::string strKey, value;

	//strKey = keyValStr.substr(0, colPos);
	//value = keyValStr.substr(colPos + 1); 
	strKey = hdrKey;
	value = hdrVal;
	
	_headerList.insert(headerMapValueType (strKey, value) );

	// calls addCookie() method to add cookie hdrs as cookie objects
	// If strKey == "cookie" then add a cookie
	if (0 == ACE_OS_String::strcasecmp(strKey.c_str(), "COOKIE") )
	{
		// Parse the cookie header value and populate the cookie list
		parseHeaderPopulateCookieList(value);
	}

}

void HttpServletRequest::parseHeaderPopulateCookieList(std::string & strCookieHdrVal)
{

	// Get the first token - version
	char eq = '=';
	const char * fieldSep = ";,";

	int eqPos =0, sepPos =0, keyStartPos = 0;
	std::string strKey, strValue;

	Cookie ckie;
	bool firstIteration = true;
	while ( sepPos != std::string::npos && (eqPos = strCookieHdrVal.find_first_of(eq, sepPos)) != std::string::npos)
	{
		strKey = strCookieHdrVal.substr(keyStartPos, eqPos - keyStartPos);

		// Remove any leading blanks in the key name
		int firstNonBlankPos = strKey.find_first_not_of(' ');
		if(firstNonBlankPos != std::string::npos)
			strKey = strKey.substr(firstNonBlankPos);

		// Find the next "," or ";"
		sepPos = strCookieHdrVal.find_first_of(fieldSep, eqPos);
		if(sepPos == std::string::npos)
		{
			strValue = strCookieHdrVal.substr(eqPos + 1);
			// no need to break now will exit in the next while
			// condition as sepPos is now npos
		}
		else
		{
			strValue = strCookieHdrVal.substr(eqPos + 1, (sepPos - eqPos - 1) );
			// set the keyStartPos to a non space character
			// after sepPos
			keyStartPos = sepPos + 1;
		}

		if(strKey[0] != '$' )
		{
			if(!firstIteration)
			{
				addCookie(ckie);
				ckie.setName("");
			}
			else
			    firstIteration = false;

			// New Cookie's $ starting info
			ckie.setName(strKey.c_str() );
			ckie.setValue(strValue.c_str() );
			

			// to set the session Id if cookie name is
			// sessionId
			if(strKey == SESSION_ID_COOKIE_NAME)
			{
				_sessionId = strValue;
			}
		}
		else
		{
			if(ACE_OS_String::strcasecmp(strKey.c_str(),"$version") == 0)
			{
				// we got the $version string so set it
				ckie.setVersion(ACE_OS::atoi(strValue.c_str() ) );
			}
			else if(ACE_OS_String::strcasecmp(strKey.c_str(), "$Path") == 0 )
			{
				// set the value in the object
				ckie.setPath( strValue);
			}
			else if(ACE_OS_String::strcasecmp(strKey.c_str(), "$Domain") == 0)
			{
				ckie.setDomain(strValue);
			} 

		} // else 

	}// while


	// add the one which did not get added in the loop
	// after checking that the name is not blank 
	// to take care of no 
	if(ckie.getName() != "")
	{
		addCookie(ckie);
	}
	
}

// searches for the name header and returns it as date (long)
long HttpServletRequest::getDateHeader(std::string name)
{
	std::string timeStr = getHeader(name);
	if(timeStr == "")
		return -1;
	else
	{
		// TO DO
		// convert it to time and return
		// timeStr.

	// TO DO convert the time as per the below  format 
	// Weekday, DD-Mon-YY HH:MM:SS TIMEZONE
	// use the ACE time functions  
	// as the time.h fns time_t, sruct tm may not be re-entrant

		return 0;
	}
}

std::string HttpServletRequest::getHeader(std::string name)
{
		if( _headerList.end() != _headerList.find(name) )
		{
			return  _headerList[name];
		}
		else
		{
			return "";
		}
}

std::vector<std::string> HttpServletRequest::getHeaders(std::string name)
{
	std::vector<std::string> hdrValues;
	headerMap::iterator mapIter = _headerList.begin();
	while (mapIter !=  _headerList.end() )
	{
		if( name == (*mapIter).first )
		{

			// It is so hard !!! to find any documentation 
			// for the std::iterator
			hdrValues.push_back((*mapIter).second );
		}
		mapIter++;
	}
	return 	hdrValues;
}

std::vector<std::string> HttpServletRequest::getHeaderNames()
{
	std::vector<std::string> hdrNames;
	headerMap::iterator mapIter = _headerList.begin();
	while (mapIter !=  _headerList.end() )
	{
		// It is so hard !!! to find any documentation 
		// for the std::iterator
		hdrNames.push_back((*mapIter).first );
		mapIter++;
	}
	return 	hdrNames;
}

int HttpServletRequest::getIntHeader(std::string name)
{
	std::string timeStr = getHeader(name);
	if(timeStr == "")
		return -1;
	else
	{
		// TO DO
		// convert it to time and return
		// timeStr.
		int intHdrVal = atoi(timeStr.c_str() );
		return intHdrVal;
	}
}

HttpSession & HttpServletRequest::getSession(bool create)
{
	if (_httpSessionObj != NULL)
	{
		return *_httpSessionObj;
	}

	if(create == false)
	{	
		throw new ServletException("NULL Session Object");
	}
	else
	{
		// Creating the session here and setting a flag
		// The servlet container should check the flag value
		// to figure out if a new session was created.
		// If there is no other error(till ServletResponse is 
		// commited returned the container will
		// save the session after generating the session Id and setting 
		// it onto this object. 
		_httpSessionObj = new HttpSession();
		_httpSessionObj->setMaxInactiveInterval(MAX_INACTIVE_INTERVAL);
		_newSessionCreated = true;
		_httpSessionObj->setId(_sessionId.c_str());
		
		// set the session id cookie
		Cookie ckie(SESSION_ID_COOKIE_NAME, _sessionId.c_str() );
		ckie.setComment("Cookie used for setting session Id");
		ckie.setPath(getContextPath().c_str() );
		ckie.setMaxAge(MAX_INACTIVE_INTERVAL);

		if(_httpServletResponseRef)
		{
			_httpServletResponseRef->addCookie(ckie);
		}
		else
		{
			throw new ServletException("HttpServletRequest does not have a reference \
									  corresponding HttpServletResponse object");
		}

		return *_httpSessionObj;
	}
}

// ADD  CODE



bool HttpServletRequest::isUserInRole(std::string role)
{
	return false;
}

std::string HttpServletRequest::getUserPrincipal()
{
	return "";
}


// MODIFY THIS METHOD TO RETURN A PROPERLY INITIALIZED/FILLED
// INPUT STREAM
ServletInputStream & HttpServletRequest::getInputStream() throw (ServletException)
{
	return _httpServletIStream;
}

