/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *
 * The code in this file was originally derived from the 
 * Java source code of The Jakarta Project (Tomcat), 
 * which is copyrighted by The Apache Software Foundation (Their 
 * copyright notice shown below). 
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * The original code may have completely changed while converting
 * to C++. It should be noted that only the packages that contain 
 * the Java Servlet API interfaces, namely javax.servlet and 
 * javax.servlet.http of the Jakarta Project are used for 
 * implementing C++WSP. The rest of the code including all the 
 * modifications to the files derived from these two packages is 
 * copyrighted by Infosys Technologies Ltd.  
 *
 */

#ifndef HTTP_SERVLET_REQUEST_H
#define HTTP_SERVLET_REQUEST_H



#include <string>
#include <vector>
#include <map>


#include "base_servlet/ServletRequest.h"
#include "Cookie.h"
#include "HttpSession.h"
#include "HttpServletResponse.h"

#include "cpp_servlet_dll.h"

static const char * const BASIC_AUTH = "BASIC";
static const char * const FORM_AUTH = "FORM";
static const char * const CLIENT_CERT_AUTH = "CLIENT_CERT";
static const char * const DIGEST_AUTH = "DIGEST";


// time in seconds for http session time out
#define MAX_INACTIVE_INTERVAL 20



typedef std::map<std::string, std::string> headerMap;
typedef headerMap::value_type headerMapValueType;

class CPP_SERVLET_DLL_API HttpServletRequest : public  ServletRequest {

   
	public:

		HttpServletRequest();

		virtual ~HttpServletRequest()
		{
			// It should not delete the HttpSession object
			// as the session cache will use the same object
		}


	// This class should implement the method from the Base class;
	 ServletInputStream & getInputStream() throw (ServletException); 


	virtual std::string getAuthType()
	{ return _authType; }
    

	virtual std::vector<Cookie> & getCookies()
	{ return _cookieList; }
    

	 // The C++ implementation will return time in seconds only unlike java
     virtual long getDateHeader(std::string name);
    
    

     virtual std::string getHeader(std::string name); 


	 virtual std::vector<std::string> getHeaders(std::string name); 
    
    

	 virtual std::vector<std::string> getHeaderNames();
    

     virtual int getIntHeader(std::string name);
    
     virtual std::string getMethod()
	 { return _method; }
    
     
     virtual std::string getPathInfo()
	 { return _pathInfo; }
    

     virtual std::string getPathTranslated()
	 { return _pathTranslated; } 
    
	 
     virtual std::string getContextPath()
	 { return _contextPath; } 
    
    

     virtual std::string getQueryString()
	 { return _queryParams; }
    

     virtual std::string getRemoteUser()
	 { return _userName; }
    

     virtual bool isUserInRole(std::string role);
    

	 // May not be relevant for the current C++ implementation 
	 virtual std::string getUserPrincipal();
    

     virtual std::string getRequestedSessionId()
	 { return _sessionId; }
    
    

     virtual std::string getRequestURI()
	 { return _requestURI; }
    
	 // Check how a string buffer is beneficial here - K Naqvi
	 // Is it only because string buffers are better in performance ?? 
	 // As of now just returning a string anyway. Shall see later
	 virtual std::string getRequestURL()
	 { return _requestURL; }
    


	 // Looks to be the same as requestURL
     virtual std::string getServletPath()
     { return _servletPath; } 
    
    
	 // Returning by reference so not to suggest that it should be freed 
	 // out side
     virtual HttpSession & getSession(bool create);
	
    
    
    
   

	 // Returning by reference so not to suggest that it should be freed 
	 // out side
     virtual HttpSession & getSession()
	 {
		 return *_httpSessionObj;
	 }
    
    
    

     virtual bool isRequestedSessionIdValid()
	 { return _oldSessionValid; }
    
    
    

     virtual bool isRequestedSessionIdFromCookie()
	 {
		 // we implement only cookie based sessions
		 // so true
		 return true;
	 }
    
    
     virtual bool isRequestedSessionIdFromURL()
	 {
		 // we implement only cookie based sessions
		 // so false
		 return false;
	 }

	 std::string getRequestLine()
	 { return _requestLine; }

	 // All the set methods

	 void setAuthType(const char * authType)
	 { _authType = authType; }

	 void setMethod (const char * meth)
	 {
		 _method = meth;
	 }

	 // parse and store header
	 void addHeader(const char * hdrKey, const char *hdrVal);

	 void addCookie(Cookie & ckie)
	 {
		_cookieList.push_back(ckie);

	 }

	 void setRequestLine(const char *reqLine)
	 {
		 _requestLine = reqLine;
	 }

	 void setQueryParams (const char *queryParams)
	 {
		 _queryParams = queryParams;
	 }
	 

	 void setUser(const char * userName)
	 {
		 _userName = userName;
	 }

	 void setSessionId(const char * sessionId)
	 {
		 _sessionId = sessionId;
	 }

	 // The space for HttpSession object is allocated by the 
	 // servlet container. So no attempt to free it should be made
	 // In this piece of the code or in the servlets.
	 void setHttpSessionObj(HttpSession * httpSession)
	 {
		 _httpSessionObj = httpSession;
	 }

	 bool newSessionCreated()
	 { return _newSessionCreated; }

	 void setOldSessionValid(bool oldSessionValid)
	 { _oldSessionValid = oldSessionValid; }

	 void setPathInfo(const char *pathInfo)
	 { _pathInfo = pathInfo; }

	 void setPathTranslated(const char *pathTransalated)
	 { _pathTranslated = pathTransalated; } 


	 // Will be set by the servlet container depending on 
	 // which servlet is getting called
	 void setContextPath(const char * ctxtPath)
	 { _contextPath = ctxtPath; }

	 void setRequestURI(const char * requestURI)
	 { _requestURI = requestURI; }

	 void setRequestURL(const char * requestURL)
	 { _requestURL = requestURL; }

	 void setServletPath(const char * servletPath)
	 { _servletPath = servletPath; }

	 void setHttpServletResponseObjRef(HttpServletResponse *httpRespRef)
	 { _httpServletResponseRef = httpRespRef; }

	 private:

		 void parseHeaderPopulateCookieList(std::string & strCookieHdrVal);

		 std::string _authType;

		 std::string _method; // GET, POST etc.

		 std::map< std::string, std::string> _headerList;
    
		 std::vector<Cookie> _cookieList;

		 std::string _pathInfo;
		 std::string _pathTranslated;

		 std::string _contextPath;

		 std::string _requestLine;

		 std::string _queryParams;

		 std::string _userName;

		 std::string _sessionId;

		 std::string _requestURI;
		 std::string _requestURL;
		 std::string _servletPath; // looks to be same as _requestURL;

		 HttpSession *_httpSessionObj;

		 // reference to the corresponding response object
		 HttpServletResponse *_httpServletResponseRef;

		 bool _newSessionCreated;

		 // set by the servlet container
		 bool _oldSessionValid;

    
		 // For now dummy Input Stream convert it to proper one 
		 // to support file upload - multipart/form-data post 
		 ServletInputStream _httpServletIStream;



    
};

#endif // HTTP_SERVLET_REQUEST_H
