/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#pragma warning ( disable : 4786 )

#include "HttpServletResponse.h"

#define SET_COOKIE_HDR_KEY "Set-Cookie"
#define COOKIE_COMMENT_KEY "Comment"
#define COOKIE_DOMAIN_KEY  "Domain"
#define COOKIE_MAXAGE_KEY  "Max-Age"
#define COOKIE_PATH_KEY     "Path"
#define COOKIE_SECURE_VALUE "Secure"
#define COOKIE_VERSION_KEY  "Version"

#define CONTENT_LENGTH_HDR_KEY "Content-Length"
#define CONTENT_TYPE_HDR_KEY   "Content-Type"

#define SESSION_ID_KEY    "sessionId"

void HttpServletResponse::addCookie(Cookie cookie)
{
	// add name value pair
	_cookieList.push_back(cookie);

}

void HttpServletResponse::addCookiesToHeaders()
{
	// Can not pass the values in quotes - "" as have to deal 
	// with browsers IE 5.0.x and Netscape 4.7x which apparently 
	// do not support new cookies

	int numCookies = _cookieList.size();
	std::string cookieStr;
	for(int i=0; i< numCookies; i++)
	{
		Cookie cookie = _cookieList[i];
		if(i != 0)
			cookieStr += "; ";

#ifdef OLD_COOKIE
		// set the name=value
		cookieStr += cookie.getName();
		

		// cookieStr += "=\"" + cookie.getValue();
		//cookieStr += "\"";
		cookieStr += "=" + cookie.getValue();
	
		// Ignoring the values of expiry time

		// Ignoring the Path value 
		// as the old cookie syntax is
		// Set-Cookie : name1=value1;[name2=value2];Path=/;expires=<expiry time>
		// So not sure which one to use. 
		// The default path is / so ok
		// The default expiry time is end of session

		
#else if // NEW _COOKIE

	// add name and value
	cookieStr += cookie.getName();
		

		// cookieStr += "=\"" + cookie.getValue();
		//cookieStr += "\"";
		cookieStr += "=" + cookie.getValue();
	// add comment
	std::string tempStr = cookie.getComment();
	if(tempStr != "")
	{
		cookieStr += ";";
		cookieStr += COOKIE_COMMENT_KEY;
		cookieStr += "=" + tempStr;
	}

	// add Domain
	tempStr = cookie.getDomain();
	if(tempStr != "")
	{
		cookieStr += ";";
		cookieStr += COOKIE_DOMAIN_KEY;
		cookieStr += "=" + tempStr;
	}

	// add Max-Age
	char maxAgeStr[50];
	sprintf(maxAgeStr, "%d", cookie.getMaxAge());
	tempStr = maxAgeStr;
	if(tempStr != "")
	{
		cookieStr += ";";
		cookieStr += COOKIE_MAXAGE_KEY;
		cookieStr += "=" + tempStr;
	}

	// add Path
	tempStr = cookie.getPath();
	if(tempStr != "")
	{
		cookieStr += ";";
		cookieStr += COOKIE_PATH_KEY;
		cookieStr += "=" + tempStr;
	}

	// add "Secure" or "" (not)
	bool secureCookie = cookie.getSecure();
	if(secureCookie)
	{
		cookieStr += ";";
		cookieStr += COOKIE_SECURE_VALUE;
	}

	// add Version
	int cookieVersion = cookie.getVersion();
	char verStr[20];
	sprintf(verStr, "%d", cookieVersion);
	tempStr = verStr;
	if(tempStr != "")
	{
		cookieStr += ";";
		cookieStr += COOKIE_VERSION_KEY;
		cookieStr += "=" + tempStr;
	}
	
#endif // elif !OLD_COOKIE
	} // for

	_servRespHdrList.insert(servRespHeaderMap::value_type(SET_COOKIE_HDR_KEY, cookieStr) );
}

bool HttpServletResponse::containsHeader(std::string name)
{
	// servRespHeaderMap _servRespHdrList;
	if( _servRespHdrList.end() != _servRespHdrList.find(name) )
	{
		return  true;
	}
	else
	{
		return false;
	}
}

// Since we shall support sessionId based on cookies only 
// not doing anything to the passed url
std::string HttpServletResponse::encodeURL(std::string url)
{
	return url;
}

// Since we shall support sessionId based on cookies only 
// not doing anything to the passed url
std::string HttpServletResponse::encodeRedirectURL(std::string url)
{
	return url;
}

void HttpServletResponse::clear()
{
	getOutputStream().clear();
	_contentLength = 0;
}


// Date format based on RFC 850 
// Weekday, DD-Mon-YY HH:MM:SS TIMEZONE
// e.g. Thursday, 06-Jun-02 12:07:30 IST
void HttpServletResponse::setDateHeader(std::string name, long date)
{
	// Just converting the long value to string and 
	// setting (to preserve the number and avoid format confusion)

	// TO DO convert the time as per the below  format 
	// Weekday, DD-Mon-YY HH:MM:SS TIMEZONE
	// use the ACE time functions  
	// as the time.h fns time_t, sruct tm may not be re-entrant


	char dateVal[50];
	sprintf(dateVal, "%d", date);
	setHeader(name, dateVal);
}

void HttpServletResponse::addDateHeader(std::string name, long date)
{
	// Just converting the long value to string and 
	// setting (to preserve the number and avoid format confusion)
	char dateVal[50];
	sprintf(dateVal, "%d", date);
	addHeader(name, dateVal);
}

void HttpServletResponse::setHeader(std::string name, std::string value)
{
	if(containsHeader(name) )
	{
		_servRespHdrList[name] = value;
	}
}

int HttpServletResponse::initializeMapIter()
{
	_servMapIter = _servRespHdrList.begin();
	if(_servMapIter == _servRespHdrList.end() )
		return -1;
	else
		return 0;
}



int HttpServletResponse::getNextHeader(std::string &key, std::string &val)
{
	if(_servMapIter == _servRespHdrList.end() )
		return -1;

	key = (*_servMapIter).first;
	val = (*_servMapIter).second;
	_servMapIter++;
	return 0;
}



void HttpServletResponse::addHeader(std::string name, std::string value)
{
	_servRespHdrList.insert(servRespHeaderMap::value_type(name, value) );
}

void HttpServletResponse::setIntHeader(std::string name, int value)
{
	char intVal[50];
	sprintf(intVal, "%d", value);
	setHeader(name, intVal);
}

void HttpServletResponse::addIntHeader(std::string name, int value)
{
	char intVal[50];
	sprintf(intVal, "%d", value);
	addHeader(name, intVal);
}


void HttpServletResponse::setSessionIdAsCookie()
{
	if(_sessionId == "")
		return;

	Cookie sessionCookie(SESSION_ID_KEY, _sessionId);
	sessionCookie.setComment("This sets the sessionId in the cookie");
	addCookie(sessionCookie);
}

void HttpServletResponse::setContentLength(int len)
{
	_contentLength = len;
	char lenStr[50];
	sprintf(lenStr, "%d", len);
	addHeader(CONTENT_LENGTH_HDR_KEY, lenStr); 
}

void HttpServletResponse::setContentType(std::string type)
{
	_contentType = type;
	addHeader(CONTENT_TYPE_HDR_KEY, type);
}

void HttpServletResponse::flushBuffer() throw(ServletException)
{
	// ADD CODE after implementing the ServletOutpuStream 
	// class (basically needs to call a flush method in the 
	// ServletOutputStream

	// Do we need to take care here if the headers have been sent
	// before this - Yes
	if(! _headersSent)
	{
		sendHeaders();
	}

	if(_servletOpStream == NULL)
	{
		ServletException excp("Servlet Output Stream NULL");
		throw excp;
	}
	else
	{
		_servletOpStream->flush();
	}
}

void HttpServletResponse::resetBuffer() throw(ServletException)
{
	if(_responseCommited)
	{
		ServletException servExcep("Response already commited");
		throw servExcep;
	}
	// ADD CODE to reset any response body 
	// may be call a method on the ServletOutputStream


}


void HttpServletResponse::reset() throw(ServletException)
{
	if(_responseCommited)
	{
		ServletException servExcep("Response already commited");
		throw servExcep;
	}

	// remove all the headers which have been added
	_servRespHdrList.clear();
	_contentLength=0;
	_contentType = "";
	_requestStatus = 0;

	// ADD CODE to reset any response body 
	// may be call a method on the ServletOutputStream


}

void HttpServletResponse::setLocale(std::string loc)
{
	_locale = loc;


}

/*
 * See RFC 2068 for the meaning of the Server status codes below.
 */

    

int HttpServletResponse::SC_CONTINUE = 100;

    
int HttpServletResponse::SC_SWITCHING_PROTOCOLS = 101;


int HttpServletResponse::SC_OK = 200;


int HttpServletResponse::SC_CREATED = 201;


int HttpServletResponse::SC_ACCEPTED = 202;


int HttpServletResponse::SC_NON_AUTHORITATIVE_INFORMATION = 203;


int HttpServletResponse::SC_NO_CONTENT = 204;


int HttpServletResponse::SC_RESET_CONTENT = 205;


int HttpServletResponse::SC_PARTIAL_CONTENT = 206;


int HttpServletResponse::SC_MULTIPLE_CHOICES = 300;


int HttpServletResponse::SC_MOVED_PERMANENTLY = 301;


int HttpServletResponse::SC_MOVED_TEMPORARILY = 302;


int HttpServletResponse::SC_SEE_OTHER = 303;


int HttpServletResponse::SC_NOT_MODIFIED = 304;


int HttpServletResponse::SC_USE_PROXY = 305;


int HttpServletResponse::SC_TEMPORARY_REDIRECT = 307;


int HttpServletResponse::SC_BAD_REQUEST = 400;


int HttpServletResponse::SC_UNAUTHORIZED = 401;


int HttpServletResponse::SC_PAYMENT_REQUIRED = 402;


int HttpServletResponse::SC_FORBIDDEN = 403;


int HttpServletResponse::SC_NOT_FOUND = 404;


int HttpServletResponse::SC_METHOD_NOT_ALLOWED = 405;


int HttpServletResponse::SC_NOT_ACCEPTABLE = 406;


int HttpServletResponse::SC_PROXY_AUTHENTICATION_REQUIRED = 407;


int HttpServletResponse::SC_REQUEST_TIMEOUT = 408;


int HttpServletResponse::SC_CONFLICT = 409;


int HttpServletResponse::SC_GONE = 410;


int HttpServletResponse::SC_LENGTH_REQUIRED = 411;


int HttpServletResponse::SC_PRECONDITION_FAILED = 412;


int HttpServletResponse::SC_REQUEST_ENTITY_TOO_LARGE = 413;


int HttpServletResponse::SC_REQUEST_URI_TOO_LONG = 414;


int HttpServletResponse::SC_UNSUPPORTED_MEDIA_TYPE = 415;


int HttpServletResponse::SC_REQUESTED_RANGE_NOT_SATISFIABLE = 416;


int HttpServletResponse::SC_EXPECTATION_FAILED = 417;


int HttpServletResponse::SC_INTERNAL_SERVER_ERROR = 500;


int HttpServletResponse::SC_NOT_IMPLEMENTED = 501;


int HttpServletResponse::SC_BAD_GATEWAY = 502;


int HttpServletResponse::SC_SERVICE_UNAVAILABLE = 503;


int HttpServletResponse::SC_GATEWAY_TIMEOUT = 504;


int HttpServletResponse::SC_HTTP_VERSION_NOT_SUPPORTED = 505;