/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *
 * The code in this file was originally derived from the 
 * Java source code of The Jakarta Project (Tomcat), 
 * which is copyrighted by The Apache Software Foundation (Their 
 * copyright notice shown below). 
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * The original code may have completely changed while converting
 * to C++. It should be noted that only the packages that contain 
 * the Java Servlet API interfaces, namely javax.servlet and 
 * javax.servlet.http of the Jakarta Project are used for 
 * implementing C++WSP. The rest of the code including all the 
 * modifications to the files derived from these two packages is 
 * copyrighted by Infosys Technologies Ltd.
 */

#ifndef HTTP_SERVLET_RESPONSE_H
#define HTTP_SERVLET_RESPONSE_H



#pragma warning ( disable : 4251 )

#include <map>

#include "base_servlet/ServletResponse.h"
#include "base_servlet/ServletException.h"
#include "Cookie.h"

#include "cpp_servlet_dll.h"


typedef std::map<std::string, std::string> servRespHeaderMap;

class CPP_SERVLET_DLL_API HttpServletResponse : public  ServletResponse {

	protected:
	
		void addCookiesToHeaders();

		servRespHeaderMap _servRespHdrList;

		servRespHeaderMap::iterator _servMapIter;

		std::vector<Cookie> _cookieList;

		int _requestStatus;


		// set by the servlet container
		std::string _sessionId;

		// Add the sessionId as a cookie
		// (Called by the servlet container before
		// flushing off a successful (status success) response)
		void setSessionIdAsCookie();

		std::string _url;

		bool _headersSent;

	public:

		HttpServletResponse():ServletResponse(), _headersSent(false)
		{}

		void setSessionId(const char *sessionId)
		{ _sessionId = sessionId; }

		std::string getSessionId()
		{ return _sessionId; }

		void setUrl(const char *url)
		{ _url = url; } 


		// This method should be implemented by a class
		// derived specific to a web server. 
		// Just like the one needed for ServletOutputStream
		virtual void sendHeaders() throw(ServletException)=0;

		void clear();

		// methods which are inherited from the base class
		// and which are over-ridden here
		void setContentLength(int len);
		void setContentType(std::string type);
		void flushBuffer() throw(ServletException);
		void resetBuffer() throw(ServletException);
		void reset() throw(ServletException);
		void setLocale(std::string loc);





	void addCookie(Cookie cookie);


     bool containsHeader(std::string name);

     std::string encodeURL(std::string url);


     std::string encodeRedirectURL(std::string url);

   
     virtual void sendError(int sc, std::string msg) throw( ServletException ) = 0;


     virtual void sendError(int sc) throw( ServletException) = 0;


     virtual void sendRedirect(std::string location) throw (ServletException) = 0;
    

     void setDateHeader(std::string name, long date);
    

	 // all time is in seconds for this C++ version
     void addDateHeader(std::string name, long date);
    

     void setHeader(std::string name, std::string value);
    

     void addHeader(std::string name, std::string value);

	 // All these methods are added because map should not be shared 
	 // between objects in different dlls
	 int initializeMapIter();

	 int getNextHeader(std::string &key, std::string &val);


     void setIntHeader(std::string name, int value);


     void addIntHeader(std::string name, int value);


    

     virtual void setStatus(int sc) = 0;
  

     static int SC_CONTINUE;

    

     static int SC_SWITCHING_PROTOCOLS ;


     static int SC_OK ;


     static int SC_CREATED;


     static int SC_ACCEPTED ;


     static int SC_NON_AUTHORITATIVE_INFORMATION ;

     static int SC_NO_CONTENT;


     static int SC_RESET_CONTENT;


     static int SC_PARTIAL_CONTENT ;


     static int SC_MULTIPLE_CHOICES;


     static int SC_MOVED_PERMANENTLY ;


     static int SC_MOVED_TEMPORARILY;


     static int SC_SEE_OTHER ;


     static int SC_NOT_MODIFIED;


     static int SC_USE_PROXY;


      static int SC_TEMPORARY_REDIRECT;


     static int SC_BAD_REQUEST;


     static int SC_UNAUTHORIZED;


     static int SC_PAYMENT_REQUIRED;


     static int SC_FORBIDDEN ;


     static int SC_NOT_FOUND;


     static int SC_METHOD_NOT_ALLOWED;


     static int SC_NOT_ACCEPTABLE;


     static int SC_PROXY_AUTHENTICATION_REQUIRED;


     static int SC_REQUEST_TIMEOUT;


     static int SC_CONFLICT;


     static int SC_GONE;


     static int SC_LENGTH_REQUIRED;


     static int SC_PRECONDITION_FAILED ;


     static int SC_REQUEST_ENTITY_TOO_LARGE;


     static int SC_REQUEST_URI_TOO_LONG;


     static int SC_UNSUPPORTED_MEDIA_TYPE;


     static int SC_REQUESTED_RANGE_NOT_SATISFIABLE ;


     static int SC_EXPECTATION_FAILED ;


     static int SC_INTERNAL_SERVER_ERROR ;


     static int SC_NOT_IMPLEMENTED ;


     static int SC_BAD_GATEWAY ;


     static int SC_SERVICE_UNAVAILABLE ;


     static int SC_GATEWAY_TIMEOUT;


     static int SC_HTTP_VERSION_NOT_SUPPORTED;
};

#endif // HTTP_SERVLET_RESPONSE_H
