/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *
 * The code in this file was originally derived from the 
 * Java source code of The Jakarta Project (Tomcat), 
 * which is copyrighted by The Apache Software Foundation (Their 
 * copyright notice shown below). 
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * The original code may have completely changed while converting
 * to C++. It should be noted that only the packages that contain 
 * the Java Servlet API interfaces, namely javax.servlet and 
 * javax.servlet.http of the Jakarta Project are used for 
 * implementing C++WSP. The rest of the code including all the 
 * modifications to the files derived from these two packages is 
 * copyrighted by Infosys Technologies Ltd.
 */

#ifndef HTTP_SESSION_H
#define HTTP_SESSION_H



#include <string>
#include <vector>
#include <map>

#include "base_servlet/ServletContext.h"
#include "HttpSessionContext.h"

#include "cpp_servlet_dll.h"

#define SESSION_ID_COOKIE_NAME "cpp_servlet_session_id"

typedef std::map<std::string, std::string> sessionMap;

class CPP_SERVLET_DLL_API HttpSession {



	public:

	HttpSession():_isNew(true), _isValid(true)
	{}

	virtual ~HttpSession() {}

	virtual long getCreationTime()
	{ return _creationTime; } 
    
    void setCreationTime(long cT)
	{ _creationTime = cT; }
    
    

	virtual std::string getId()
	{ return _sessionId; }

	// called by servlet container
	void setId(const char * sesId)
	{ _sessionId = sesId; }
    


     virtual long getLastAccessedTime()
	 { return _lastAccessedTime; }


	 void setLastAccessesTime(long lastAccessTime)
	 { _lastAccessedTime = lastAccessTime; }
    
    

	 // May keep a reference within it
     virtual ServletContext & getServletContext()
	 { return *_applicationCtxt; }


    
     virtual void setMaxInactiveInterval(int interval)
	 { _maxInactiveInterval = interval; }





     virtual int getMaxInactiveInterval()
	 { return _maxInactiveInterval; }
    
    


  
	// Not supporting objects in this C++ implementation for now.
	// Later on may support objects which implement a certain interface - K Naqvi
	virtual std::string getAttribute(std::string name);
    
    
    
    
  
	 std::string getValue(std::string name)
	 { return getAttribute(name); } 
    
    
    

    
	 virtual std::vector<std::string> getAttributeNames();
    
    
    

    
	  std::vector<std::string> getValueNames()
	  { return getAttributeNames(); }
    
    
    
 
     virtual void setAttribute(const char * name, const char * value);
    


 
     void putValue(const char * name, const char * value)
	 { setAttribute(name, value); }



     virtual void removeAttribute(const char * name);



     void removeValue(const char * name)
	 { removeAttribute(name); }


     virtual void invalidate()
	 { _isValid = false; } // really very thin implementation 
	                       // in the C++ version as not implementing 
	                       // binding/unbinding for now. 
    
	 // Adding a method which is not there in java counter part
	 // perhaps can help in delayed destruction (batching) of the session 
	 // object. 
	 bool isValid()
	 { return _isValid; }


     virtual bool isNew()
	 { return _isNew; }

	 void setOld()
	 { _isNew = false; }

	 private:

		 long _creationTime;
		 bool _isNew;
		 std::string _sessionId;
		 long _lastAccessedTime;
		 int _maxInactiveInterval;

		 bool _isValid;

		 ServletContext *_applicationCtxt; // Don't delete just keep a reference

		 sessionMap _sessParamList;
};

#endif // HTTP_SESSION_H

