/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include "ace\Date_Time.h"
#include "ace\High_Res_Timer.h"
#include "MessageLogger.h"

char* MessageLogger::_logFile = "C:/cpp_servlet_log.txt";
std::ofstream MessageLogger::_logStream("C:/cpp_servlet_log.txt", ofstream::app);
eLogLevel MessageLogger::_logLevel = eLogDebug;
ACE_High_Res_Timer MessageLogger::timer;

MessageLogger::MessageLogger()
{
	timer.start();
}

MessageLogger::~MessageLogger()
{
	if (_logStream.is_open())
	{
		_logStream << "Closing the file\n";
		_logStream.close();
	}
}

void MessageLogger::startTimer()
{
	timer.start();
}
void MessageLogger::stopTimer()
{
	timer.stop();
}
void MessageLogger::getElapsedTime (ACE_Time_Value &tv)
{
	timer.stop();
	timer.elapsed_time (tv);
	timer.start();
}

void MessageLogger::setLogFile (char* logFile)
{
	_logFile = logFile;
	
	if (_logStream.is_open())	
		_logStream.close();

	_logStream.open (_logFile, ofstream::app);
	/*
	std::ofstream logStream(logFile, ofstream::app);
	_logStream = logStream;
	*/
}

void MessageLogger::setLogLevel (eLogLevel logLevel)
{
	_logLevel = logLevel;
}

void MessageLogger::LogServletLog (const char *contextPath, const char *message)
{
	ACE_Date_Time aceDateTime;
	std::string logMessage;
	char buffer[4];

	logMessage.append("[");
	_ltoa(aceDateTime.month() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append("/");
	_ltoa(aceDateTime.day() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append("/");
	_ltoa(aceDateTime.year() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append("][");
	_ltoa(aceDateTime.hour() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append(":");
	_ltoa(aceDateTime.minute() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append(":");
	_ltoa(aceDateTime.second() ,buffer, 10);
	logMessage.append(buffer);
	
	logMessage.append("]\t");

	logMessage.append(contextPath);
	logMessage.append("\t");

	logMessage.append(message);
	logMessage.append("\n");
		
	_logStream << logMessage.c_str();
	_logStream.flush();


}

void MessageLogger::LogMessage (char messageType, char *location, char *message, ...)
{

	// Returning if messageType is greater than _logLevel
	switch (messageType)
	{
	case 'D':
	case 'd':
		// Log Debug messages only if debug level is eLogDebug
		if (_logLevel < eLogDebug)
			return;
		break;
	case 'I':
	case 'i':
		// Log Info messages if debug level is either Debug or Info.
		if (_logLevel < eLogInfo)
			return;
		break;
	}

	
	ACE_Date_Time aceDateTime;
	
	std::string logMessage;
	char buffer[4];

	long		c,i,j;
	va_list		args;
	char		*fmtPtr;
	static	char		temp1[512], temp2[512];
	int			tok = '%';
	int			fld = FALSE, fmtFlg = FALSE, dec = FALSE;

	va_start(args, message);
	vsprintf(temp1, message, args);
	va_end(args);

	memset( temp2, 0, 512 ); // Clear temp2 buffer
// format spec: %[flag][width][.precision][{h|l|I64|L}]type

	fmtPtr = message;
	i = j = 0;
	while (temp1[i] != 0)
	{
		switch (*fmtPtr)
		{
		case '%':				// format spec?
			if (fmtFlg)
				fmtFlg = FALSE;	// step over %%
			else
				fmtFlg = TRUE;	// set flag
			fmtPtr++;
			break;

		case 'd':				// decimal
		case 'i':				// signed
		case 'u':				// unsigned
		case 'f':				// float
			if (fmtFlg)
			{
				dec = TRUE;
				fld = TRUE;
				fmtFlg = FALSE;	// no longer in format spec.
			}
			fmtPtr++;
			break;

		case 'c':				// these terminate the ones
		case 'C':				// we don't care about
		case 'o':
		case 'x':
		case 'X':
		case 'e':
		case 'E':
		case 'g':
		case 'G':
		case 'n':
		case 'p':
		case 's':
		case 'S':
			if (fmtFlg)
			{
				fld = TRUE;
				fmtFlg = FALSE;	// no longer in format spec.
			}
			fmtPtr++;
			break;

		default:				// everything else
			fmtPtr++;
			break;

		} // switch

		if (!fmtFlg && !fld)
			if (temp1[i] == '\n')
			{
				temp2[j++] = temp1[i++];
			}
			else
				temp2[j++] = temp1[i++];	// move char

		if (!fmtFlg && fld)
		{
			fld = FALSE;
			if (temp1[i] == '\n')
			{
				temp2[j++] = temp1[i++];
			}

			while(temp1[i] == ' ')
				temp2[j++] = temp1[i++];	// move leading spaces

			if (dec)
			{
				c = 0;
				while ((temp1[i + c] >= '0') && (temp1[i + c] <= '9'))
					c++;						// count digits

				while (c)
				{
					c--;
					temp2[j++] = temp1[i++];	// move digits
					if (c)
						if ((c % 3) == 0)
							temp2[j++] = ',';	// place comma
				}
				if (temp1[i] == '.')
				{
					temp2[j++] = temp1[i++];	// move decimal point
					while ((temp1[i] >= '0') && (temp1[i] <= '9'))
						temp2[j++] = temp1[i++];	// move digits
				}
				dec = FALSE;
			}
			else
				while ((temp1[i] != ' ') && (temp1[i] != 0))
					temp2[j++] = temp1[i++];	// move field
		}
	}

	logMessage.append("[");
	_ltoa(aceDateTime.month() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append("/");
	_ltoa(aceDateTime.day() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append("/");
	_ltoa(aceDateTime.year() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append("][");
	_ltoa(aceDateTime.hour() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append(":");
	_ltoa(aceDateTime.minute() ,buffer, 10);
	logMessage.append(buffer);
	logMessage.append(":");
	_ltoa(aceDateTime.second() ,buffer, 10);
	logMessage.append(buffer);

	logMessage.append("]\t");

	logMessage.append(location);
	logMessage.append("\t");

	logMessage.append(temp2);
	logMessage.append("\n");
	va_end (args);

	switch (messageType)
	{
	case 'D':
	case 'd':
		logMessage.insert(0, "DEBUG\t");
		_logStream << logMessage.c_str();
		break;
	case 'I':
	case 'i':
		logMessage.insert(0, "INFO\t");
		
		_logStream << logMessage.c_str();
		break;
	case 'E':
	case 'e':
		logMessage.insert(0, "ERROR\t");
		
		// Log Error messages in all the cases.
		_logStream << logMessage.c_str();
		break;


#ifdef TIMELOG

	case 'T':
	case 't':

		logMessage.insert(0, "TIMELOG\t");
		_logStream << logMessage.c_str();



		ACE_Time_Value aceTime;
		getElapsedTime (aceTime);
		//= ACE_OS::gettimeofday();
		float timeInMilliSeconds = ((float)aceTime.sec()*1000.0) + ((float)aceTime.usec()/1000.0);
		char message[100];
		sprintf (message, "Time In MilliSeconds = %f \n", timeInMilliSeconds);

		_logStream << message;

		break;

#endif


	}
	
	_logStream.flush();


}