/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#pragma warning ( disable : 4786 )

#include "ServletFactory.h"
#include "MessageLogger.h"

// Don't need this in the dll model
// void noOpLoadServlets();

// registeredFunctionsMap ServletFactory::_functionMap;
ServletFactory * ServletFactory::_TheServletFactory = NULL;

ServletFactory* ServletFactory::instance()
{
	if(_TheServletFactory== NULL)
		_TheServletFactory = new ServletFactory();

	return _TheServletFactory;
}

void ServletFactory::registerToFactory (const char* servletName, registerFunction functionName)
{

	registeredFunctionsMap::iterator regFnIter;
	regFnIter = instance()->_functionMap.find(servletName);
	if(regFnIter ==  instance()->_functionMap.end() )
	{
		// If not present insert otherwise don't do anything
		instance()->_functionMap.insert(registeredFunctionsMap::value_type(servletName, functionName) );
	}
}

HttpServlet * ServletFactory::getServlet (const char* servletName)
{
	
	//MessageLogger::LogMessage('D', "ServletFactory", "Inside ServletFactory::getServlet() \"%s\"", "");
	registeredFunctionsMap::iterator regFnIter;
	regFnIter = instance()->_functionMap.find(servletName);
	if(regFnIter !=  instance()->_functionMap.end() )
	{
		//MessageLogger::LogMessage('D', "ServletFactory", "Found servlet \"%s\"", servletName);
		registerFunction fPtr = (*regFnIter).second;
		HttpServlet * servletObj = (*fPtr)();
		// HttpServlet *servletObj = new HttpServlet;
		return servletObj;
	}
	else
	{
		return NULL;
	}
	//MessageLogger::LogMessage('D', "ServletFactory", "Leaving ServletFactory::getServlet() \"%s\"", "");
}

void ServletFactory::LoadServlets ()
{
	// Don't need this in the dll model
	/**
	noOpLoadServlets();
	**/
}