/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __SERVLET_FACTORY_H
#define __SERVLET_FACTORY_H

#include "HttpServlet.h"

// The factory class to create cpp user servlets.
#include <string>
#include <map>
#include "cpp_servlet_dll.h"

typedef  HttpServlet* (*registerFunction) (); 
typedef std::map< std::string, registerFunction> registeredFunctionsMap;
typedef registeredFunctionsMap::value_type  registeredFunctionsMapType;

//TODO

//Define the MACRO's to be used by the servlets to register
//with the factory.

//#define QUOTE ""
// #define formServletName(A, QUOTE) QUOTE##A##QUOTE

// Have to introduce this second parameter as it is very difficult to 
// use a macro parameter value in a string. So expecting somebody 
// to call REGISTER_SERVLET(TryExampleServlet, "TryExampleServlet")

#define REGISTER_SERVLET(SERVLET_CLASS_NAME, SERVLET_NAME) \
	HttpServlet *createFn##SERVLET_CLASS_NAME() \
{\
	HttpServlet *my##SERVLET_CLASS_NAME## =  new SERVLET_CLASS_NAME();\
	return my##SERVLET_CLASS_NAME; \
}\
struct dummy##SERVLET_CLASS_NAME { \
	dummy##SERVLET_CLASS_NAME() \
{\
ServletFactory::registerToFactory( SERVLET_NAME, &createFn##SERVLET_CLASS_NAME ); \
}\
};\
	struct dummy##SERVLET_CLASS_NAME dummyInstance##SERVLET_CLASS_NAME##;


// Exporting this class 
// TO DO put all such stuff under ifdef - make it portable

class CPP_SERVLET_DLL_API ServletFactory 
{
public:
	
	virtual ~ServletFactory() {}
	
	static void registerToFactory (const char* servletName, registerFunction functionName);
	static HttpServlet *getServlet (const char* servletName);

	static ServletFactory * instance();

	// The below function is used when user servlets lib is used a 
	// a dll. It will then overwrite the user_servlets_lib.dll's factory
	// with the one in this holder. 
	// This is only a temporary solution - for a permanent solution
	// the cpp_servlets_lib should be made a dll. Or atleast the 
	// ServletFactory alone should be made into a separate dll.
	static setInstance(ServletFactory * servFactory)
	{
		//if(_TheServletFactory)
		//	delete _TheServletFactory;
		if(servFactory != NULL)
			_TheServletFactory = servFactory;
	}
	
	static void LoadServlets();

private:

	 ServletFactory() {}
	 registeredFunctionsMap _functionMap;

	 // singleton instance
	 static ServletFactory *_TheServletFactory;

};





#endif // __SERVLET_FACTORY_H