/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "ServletRequestDispatcher.h"
#include "ServletFactory.h"
#include "MessageLogger.h"


ServletRequestDispatcher::ServletRequestDispatcher(const char *path):RequestDispatcher(path)
{
}


void ServletRequestDispatcher::forward(ServletRequest &request, ServletResponse &response)
	throw( ServletException)
{
	// This method supports only calling of other servlets as of now
	// take out the servlet name from the _path 
	// create the servlet from servlet factory and and invole the 
	// service method on it by passing the request and response objects by 
	// reference

	int befLastSlash=0, prevSlash=0;
	befLastSlash = _path.find_last_not_of("/\\");
	// to take care of multiple slashes occuring
	// befLastSlash = _path.find_last_not_of("/\\", befLastSlash);

	if(befLastSlash == std::string::npos)
	{
		// throw exception
		ServletException svltExcp(std::string("Illegal path given for the resource " ) );
		throw svltExcp;
	}

	prevSlash = _path.find_last_of("/\\", befLastSlash - 1);

	if(prevSlash == std::string::npos)
	{
		// throw exception
		ServletException svltExcp(std::string("Illegal path given for the resource " ) );
		throw svltExcp;
	}

	std::string cppServletName = _path.substr(prevSlash + 1, ( befLastSlash - prevSlash) );


	MessageLogger::LogMessage('D', "ServletRequestDispatcher", "Get the servlet from dll: \"%d\"", (int) ServletFactory::instance());

	// HttpServlet * servletObj = f1();
	// HttpServlet *servletObj = new TryExampleServlet();

	HttpServlet * servletObj = ServletFactory::getServlet(cppServletName.c_str());
	
	// HttpServlet * servletObj =  get_servlet_from_dll_factory(cppServletName.c_str() );
	

	if( servletObj == NULL)
	{
		MessageLogger::LogMessage('E', "ServletRequestDispatcher", "Servlet Factory could not create servlet : \"%s\"", cppServletName.c_str() );
		ServletException svltExcp(std::string("Servlet Factory could not create servlet : " + cppServletName) );
		throw svltExcp;
	}


	// Invoke the servlet's service method
	servletObj->service( request, response);
	// job is done so delete it
	delete servletObj;

}
