/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#pragma warning ( disable : 4786 )

#include "ApplicationContextManager.h"

#include <parsers\DOMParser.hpp>
#include <framework\LocalFileInputSource.hpp>
#include <util\PlatformUtils.hpp>

#include "..\MessageLogger.h"

ApplicationContextManager::ApplicationContextManager(const char *contextFileName)
{

	XMLPlatformUtils::Initialize();
	setContextFile(contextFileName);	
}


ServletContext* ApplicationContextManager::getServletContext()
{
	_servletContext = new ServletContext();
	if (_servletContext == NULL)
		return NULL;

	if (_domElement.isNull())
		return NULL;

	DOM_NodeList nodeList = _domElement.getElementsByTagName("param-name");
	DOM_Element currentNode, nextNode, parentNode;

	contextInitParamMap & contextParam = _servletContext->getContextParamMap();
	mimeMap				& contextMimeMap = _servletContext->getContextMimeMap();

	for (int i=0; i<nodeList.getLength ();i++)
	{
		currentNode = (DOM_Element &)nodeList.item(i);
		parentNode = (DOM_Element &) currentNode.getParentNode();
		nextNode = (DOM_Element &) currentNode.getNextSibling();
		nextNode = (DOM_Element &) nextNode.getNextSibling();

		if (strcmp(parentNode.getNodeName().transcode(), "context-param") == 0)
		{
			std::string paramName = currentNode.getFirstChild().getNodeValue().transcode() ;
			std::string paramValue = nextNode.getFirstChild().getNodeValue().transcode();
			//MessageLogger::LogMessage('d',"appctxtmgr", "context ParamName = %s", paramName.c_str());
			//MessageLogger::LogMessage('d',"appctxtmgr", "context ParamValue = %s", paramValue.c_str());
			paramName = StripSpaceChars(paramName);
			paramValue = StripSpaceChars(paramValue);
			contextParam [paramName] = paramValue;
		}
	}


	std::string rightAfter = _servletContext->getInitParameter("colour");
	//MessageLogger::LogMessage('d',"appctxtmgr", "context ParamValue  for colour is = %s", rightAfter.c_str());


	nodeList = _domElement.getElementsByTagName("extension");
	for (i=0; i<nodeList.getLength();i++)
	{
		currentNode = (DOM_Element &)nodeList.item(i);
		parentNode = (DOM_Element &) currentNode.getParentNode();
		nextNode = (DOM_Element &) currentNode.getNextSibling();
		nextNode = (DOM_Element &) nextNode.getNextSibling();

		if (strcmp(parentNode.getNodeName().transcode(), "mime-mapping") == 0)
		{
			std::string paramName = currentNode.getFirstChild().getNodeValue().transcode() ;
			std::string paramValue = nextNode.getFirstChild().getNodeValue().transcode();
			paramName = StripSpaceChars(paramName);
			paramValue = StripSpaceChars(paramValue);
			contextMimeMap [paramName] = paramValue;
		}
	}

	nodeList = _domElement.getElementsByTagName("session-timeout");
	currentNode = (DOM_Element &)nodeList.item(0);
	parentNode = (DOM_Element &) currentNode.getParentNode();

	if (strcmp(parentNode.getNodeName().transcode(), "session-config") == 0)
	{
		std::string timeOut = currentNode.getFirstChild().getNodeValue().transcode() ;
		timeOut = StripSpaceChars(timeOut);
		_servletContext->setSessionTimeOut(atol(timeOut.c_str()));
	}

	// Set the context name from the display-name element in the 
	// XML file if it is there. Otherwise set it to blank
	nodeList = _domElement.getElementsByTagName("display-name");
	if(nodeList.getLength() > 0)
	{
		currentNode = (DOM_Element &)nodeList.item(0);
		parentNode = (DOM_Element &) currentNode.getParentNode();

		if (strcmp(parentNode.getNodeName().transcode(), "web-app") == 0)
			_servletContext->setContextName(currentNode.getFirstChild().getNodeValue().transcode());
	}
	else
	{
			_servletContext->setContextName("");
	}

	_servletContext->setContextPath(_contextPath);

	nodeList = _domElement.getElementsByTagName("dll-name");
	std::string dllName = nodeList.item(0).getFirstChild().getNodeValue().transcode();
	dllName = StripSpaceChars(dllName);
	_servletContext->setDLLName(dllName);

	return _servletContext;
}

ServletConfigDictionary* ApplicationContextManager::getServletConfigDictionary()
{

	int i=0, j=0;
	ServletConfigDictionary *srvConfigDict = new ServletConfigDictionary();
	ServletConfig *configObject;

	if (srvConfigDict == NULL)
		return NULL;

	if (_domElement.isNull())
		return NULL;

	if (_servletContext == NULL)
		return NULL;
	
	DOM_NodeList servletList = _domElement.getElementsByTagName("servlet");
	DOM_Element currentNode;

	for (i=0; i<servletList.getLength();i++)
	{
		// Here create the configObject and populate the values.
		configObject = new ServletConfig();
		if (configObject == NULL)
			continue;
		
		// set servlet name here
		currentNode = (DOM_Element &)servletList.item(i);
		DOM_NodeList nameNodeList = currentNode.getElementsByTagName("servlet-name");
		std::string servletName = nameNodeList.item(0).getFirstChild().getNodeValue().transcode();

		servletName = StripSpaceChars(servletName);

		configObject->setServletName(servletName);
		//MessageLogger::LogMessage('d',"appctxtmgr", "servlet Name = %s", servletName.c_str());

		DOM_NodeList paramNameList = currentNode.getElementsByTagName("param-name");
		
		// get the param map object from the configObject here
		initParamMap & servletInitParam  = configObject->getConfigInitParamMap();

		for (j=0; j<paramNameList.getLength(); j++)
		{
			DOM_Element nodeParamName = (DOM_Element &)paramNameList.item(j);
			DOM_Element nodeParamValue =(DOM_Element &)nodeParamName.getNextSibling();
			nodeParamValue = (DOM_Element &) nodeParamValue.getNextSibling();

			// add the name/value pair to the map object here
			std::string paramName = nodeParamName.getFirstChild().getNodeValue().transcode();
			std::string paramValue = nodeParamValue.getFirstChild().getNodeValue().transcode();
			paramName = StripSpaceChars(paramName);
			paramValue = StripSpaceChars(paramValue);
			//MessageLogger::LogMessage('d',"appctxtmgr", "servletParamName = %s", paramName.c_str());
			//MessageLogger::LogMessage('d',"appctxtmgr", "servletParamValue = %s", paramValue.c_str());
			servletInitParam [paramName] = paramValue;
		}
		// set the reference to the cotext object in the config object here
		configObject->setServletContext(_servletContext);

		std::string rightAfter = configObject->getInitParameter("colour");
		//MessageLogger::LogMessage('d',"appctxtmgr", "servletParamValue  for colour is = %s", rightAfter.c_str());

		// add the configObject to the configDictionary here
		srvConfigDict->addServletConfigToDictionary(servletName, configObject);

	} 

	return srvConfigDict;
}


void ApplicationContextManager::setContextFile(const char *contextFileName)
{

	DOMParser		domParser;
	DOM_Document	domDocument;
	
	int iIndex=0, lastOccurance=0;
	
	char contextFile[200],	contextPath[200];
	if (contextFileName == NULL)
		return ;

	strcpy(contextFile, contextFileName);

	while(contextFileName[iIndex] != '\0')
	{
		 if (contextFileName[iIndex] == '\\')
			contextFile[iIndex] = '/';
		else
			contextFile[iIndex] = contextFileName[iIndex];
		
		if (contextFile[iIndex] == '/')
			lastOccurance = iIndex;

		iIndex++;
	}

	strncpy (contextPath, contextFile, lastOccurance);
	contextPath[lastOccurance] = '\0';

	_contextPath = contextPath;

	// Letting the default validation i.e. Val_Auto
	// It will validate only if a grammar is specified
	// domParser.setValidationScheme (DOMParser::Val_Always);
	// Use default
	domParser.setIncludeIgnorableWhitespace (false);		
	DOMString dstrFileName	(contextFileName);		
			
	LocalFileInputSource inputSource (dstrFileName.rawBuffer ());

	domParser.parse (inputSource);
	domDocument = domParser.getDocument ();
	if (domDocument.isNull ())
	{
		MessageLogger::LogMessage('d',"AppContxtMgr::setContextFileName", "Returning because domElement is NULL");
		return ;
	}
	_domElement = domDocument.getDocumentElement ();

}

std::string ApplicationContextManager::StripSpaceChars(std::string &str)
{
	int firstNonSpaceChar = str.find_first_not_of(' ');
	int lastNonSpaceChar = str.find_last_not_of(' ');
	return str.substr(firstNonSpaceChar, (lastNonSpaceChar - firstNonSpaceChar + 1) );
}

/*
std::string ApplicationContextManager::getDeploymentDLLName()
{
	if (_domElement.isNull())
		return "";

	DOM_NodeList nodeList = _domElement.getElementsByTagName("dll-name");
	std::string dllName = nodeList.item(0).getFirstChild().getNodeValue().transcode();
	return dllName;
}
*/