/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#pragma warning ( disable : 4786 )

#include "ServletConfig.h"
#include "..\MessageLogger.h"

std::string  ServletConfig::getInitParameter(const char * name)
{
	std::string paramName(name);
	std::string paramValue;

	// MessageLogger::LogMessage('d', "ServletConfig::getInitParameter", "Param Name is %s", paramName.c_str());
	if( _initParamList.end() != _initParamList.find(paramName) )
	{
		paramValue =  _initParamList[paramName];
	}
	else
	{
		paramValue = "";
	}
	// MessageLogger::LogMessage('d', "ServletConfig::getInitParameter", "Param Value is %s", paramValue.c_str());

	return paramValue;
}

std::vector<std::string> ServletConfig::getInitParameterNames()
{
	std::vector<std::string> paramNames;
	initParamMap::iterator mapIter = _initParamList.begin();
	
	while (mapIter !=  _initParamList.end() )
	{
		paramNames.push_back((*mapIter).first );
		mapIter++;
	}
	return 	paramNames;
}

void ServletConfig::setInitParameters(const char * name, const char * value)
{
	initParamMap::iterator mapIter = _initParamList.begin();
	while (mapIter !=  _initParamList.end() )
	{
		if( name == (*mapIter).first )
		{
			(*mapIter).second  = value;
			return;
		}
		mapIter++;
	}
	_initParamList.insert(initParamMap::value_type(name, value) );
}
