/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#pragma warning ( disable : 4786 )

#include <fstream>
#include "servlet_version.h"
#include "ServletContext.h"
#include "..\MessageLogger.h"
#include "ace/Dirent.h"
#include "ace/OS.h"
#include "ApplicationContextManager.h"

std::string ServletContext::getMimeType(std::string file)
{

	if (file == "")
		return "";
	int pos = file.rfind('.');
	if (pos < 0)
		return "";
	std::string extension = file.substr(pos + 1);
	if (extension.length() < 1)
		return "";

	if( _mimeMap.end() != _mimeMap.find(extension) )
	{
		return  _mimeMap[extension];
	}
	else
	{
		return "";
	}
}

// ADD CODE
std::vector<std::string> ServletContext::getResourcePaths(std::string path)
{
	std::vector<std::string> resourcePaths;
	std::string dirPath (_contextPath.c_str());
	dirPath.append(path.c_str());

	ACE_Dirent dir (ACE_TEXT (dirPath.c_str()));
	char buff[200];
	ACE_stat fileStat;
	dirent *directory;

	while ((directory = dir.read ()) != 0 )
	{
		memset(buff,0,200);
		strcat(buff, dirPath.c_str());
		strcat(buff, directory->d_name);
		
		ACE_OS::stat(buff, &fileStat);
		std::string resourceEntry;
		resourceEntry.append(path.c_str());
		resourceEntry.append(directory->d_name);

		if (fileStat.st_mode & _S_IFDIR)
			resourceEntry.append("/");
		
		resourcePaths.push_back(resourceEntry);
	}

	return resourcePaths;
}

// Need to see how this should get implemented in C++
// In java it used to return a URL object
std::string ServletContext::getResource(std::string path)
{
	return path;
}

// Very simplistic implementation (for html files
std::istream & ServletContext::getResourceAsStream(std::string path)
{
	// Check the mime type of the resource pointed to by the 
	// path and return a file stream.
	// ADD CODE to throw an exception if the type is not supported.

	std::ifstream myFileStream(path.c_str() ); // will work to read the html files
	return myFileStream;
}

// Again a very simplistic (NO-OP) kind of implementation
// Modify this method after understanding the functionaing of
// RequestDispatcher in more details
ServletRequestDispatcher* ServletContext::getRequestDispatcher(std::string path)
{
	ServletRequestDispatcher *rd = new ServletRequestDispatcher(path.c_str());
	return rd;
}

// Returns a dispatcher for a named servlet
// ADD CODE to fix the below implementation for calling 
// servlets from a servlet
ServletRequestDispatcher* ServletContext::getNamedDispatcher(std::string name)
{
	ServletRequestDispatcher *rd = new ServletRequestDispatcher(name.c_str());
	return rd;
}

// ADD CODE to log the log
void ServletContext::log(const char * msg)
{
	MessageLogger::LogServletLog(_contextName.c_str(), msg);
}

// ADD CODE to log the servlet exception
void ServletContext::log(const char * message, ServletException servExcep)
{
	std::string logMessage(message);
	logMessage.append(" ; additional information from the servlet exception=");
	logMessage.append(servExcep.getRootCause());
	MessageLogger::LogServletLog(_contextName.c_str(), logMessage.c_str());
}

// ADD CODE to get the real path (based on computer and OS ) from 
// the virtual path passed. E.g. for /index.html return 
// "D:\Naqvi\Apache\htdocs\index.html" on NT. Check the header documentation
std::string ServletContext::getRealPath(std::string path)
{
	std::string realPath = _contextPath;
	realPath.append(path.c_str());
	return realPath;
}

std::string ServletContext::getServerInfo()
{
	std::string serverInfo = SERVER_NAME;
	serverInfo += VERSION;
	return serverInfo;
}

// ADD CODE ro read and return the web application name from the 
// deployment descriptor of the application.
std::string ServletContext::getServletContextName()
{
	return _contextName;
}

std::string ServletContext::getDLLName()
{
	return _dllName;
}

std::string ServletContext::getInitParameter(std::string name)
{
	std::string paramValue;

	//MessageLogger::LogMessage('d', "ServletContext::getInitParameter", "Param Name is %s", name.c_str());
	if( _ctxtParamMap.end() != _ctxtParamMap.find(name) )
	{
		paramValue = _ctxtParamMap[name];
	}
	else
	{
		paramValue = "";
	}
	//MessageLogger::LogMessage('d', "ServletContext::getInitParameter", "Param Value is %s", paramValue.c_str());

	return paramValue;
}

void ServletContext::setInitParameter(const char *name, const char *value)
{
	contextInitParamMap::iterator mapIter = _ctxtParamMap.begin();
	while (mapIter !=  _ctxtParamMap.end() )
	{
		if( name == (*mapIter).first )
		{
			// It is so hard !!! to find any documentation 
			// for the std::iterator
			(*mapIter).second  = value;
			return;
		}
		mapIter++;
	}
	_ctxtParamMap.insert(contextInitParamMap::value_type(name, value) );
}

std::vector<std::string> ServletContext::getInitParameterNames()
{
	std::vector<std::string> paramNames;
	contextInitParamMap::iterator mapIter = _ctxtParamMap.begin();
	
	while (mapIter !=  _ctxtParamMap.end() )
	{
		paramNames.push_back((*mapIter).first );
		mapIter++;
	}
	return 	paramNames;
}

ServletContext ServletContext::getContext(std::string uripath)
{
	// Should throw an exception in this case
	// if (uripath == "")
	//	return NULL;

	ServletContext *srvContext;

	int pos = _contextPath.rfind('/');
	std::string configFilePath = _contextPath.substr(0, pos);

	configFilePath.append(uripath);
	configFilePath.append("/web.xml");

	ApplicationContextManager appContextMgr(configFilePath.c_str());
	srvContext = appContextMgr.getServletContext();

	return (*srvContext);

}
