/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *
 * The code in this file was originally derived from the 
 * Java source code of The Jakarta Project (Tomcat), 
 * which is copyrighted by The Apache Software Foundation (Their 
 * copyright notice shown below). 
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * The original code may have completely changed while converting
 * to C++. It should be noted that only the packages that contain 
 * the Java Servlet API interfaces, namely javax.servlet and 
 * javax.servlet.http of the Jakarta Project are used for 
 * implementing C++WSP. The rest of the code including all the 
 * modifications to the files derived from these two packages is 
 * copyrighted by Infosys Technologies Ltd.  
 */

#ifndef SERVLET_CONTEXT_H
#define SERVLET_CONTEXT_H

#pragma warning ( disable : 4251 )

#include <string>
#include <vector>
#include <map>
#include <iostream>

#include "ServletException.h"
#include "../ServletRequestDispatcher.h"

#include "../cpp_servlet_dll.h"

typedef std::map<std::string, std::string> contextInitParamMap;
typedef std::map<std::string, std::string> mimeMap;

class CPP_SERVLET_DLL_API ServletContext {

	public:


	
	ServletContext getContext(std::string uripath);
    
    


     std::string getMimeType(std::string file);
    
        
	 // Check the implementation of this method in the java version
	 std::vector<std::string> getResourcePaths(std::string path);
    
    
	 // In the java version it returns a URL object
	 std::string getResource(std::string path); //throws ServletException ;
    
    
	 std::istream & getResourceAsStream(std::string path);
    

     ServletRequestDispatcher* getRequestDispatcher(std::string path);


     ServletRequestDispatcher* getNamedDispatcher(std::string name);
    
        
     void log(const char * msg);
    
    
     void log(const char * message, ServletException servExcep);
    
    
    

     std::string getRealPath(std::string path);
    
    
     std::string getServerInfo();
    

     std::string getInitParameter(std::string name);
    
    
	 // Used by the servlet container when it loads an application 
	 // context. 
	 // The servlet container would typically read these values from 
	 // the web application's deployment descriptor file.
	 void setInitParameter(const char *name, const char *value);


     std::vector<std::string> getInitParameterNames();
    
    

	 // Commenting out since don't have a java like base Object in C++.
	 // May go in for it by having a Attribute interface later
	 // std::string getAttribute(std::string name);
       

	 // Commenting out since don't have a java like base Object in C++.
	 // May go in for it by having a Attribute interface later
     // std::vector<std::string> getAttributeNames();
        
	 // Commenting out since don't have a java like base Object in C++.
	 // May go in for it by having a Attribute interface later
     // void setAttribute(std::string name, std::string value);
    
	 // Commenting out since don't have a java like base Object in C++.
	 // May go in for it by having a Attribute interface later
     // void removeAttribute(std::string name);
    
    
	 std::string getServletContextName();
	 std::string getDLLName();

	 private:

	 contextInitParamMap & getContextParamMap()
	 {
		 return _ctxtParamMap;
	 }
	 mimeMap & getContextMimeMap()
	 {
		 return _mimeMap;
	 }

	 void setSessionTimeOut (long timeOut)
	 {
		 _sessionTimeOut = timeOut;
	 }

	 void setContextName (std::string name)
	 {
		 _contextName = name;
	 }

	 void setContextPath (std::string path)
	 {
		 _contextPath = path;
	 }

	 void setDLLName (std::string name)
	 {
		 _dllName = name;
	 }

	 contextInitParamMap	_ctxtParamMap;
	 mimeMap				_mimeMap;
	 long					_sessionTimeOut;
	 std::string			_contextName;
	 std::string			_contextPath;
	 std::string			_dllName;
	 friend class ApplicationContextManager;

};

#endif // SERVLET_CONTEXT_H


