/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *
 * The code in this file was originally derived from the 
 * Java source code of The Jakarta Project (Tomcat), 
 * which is copyrighted by The Apache Software Foundation (Their 
 * copyright notice shown below). 
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * The original code may have completely changed while converting
 * to C++. It should be noted that only the packages that contain 
 * the Java Servlet API interfaces, namely javax.servlet and 
 * javax.servlet.http of the Jakarta Project are used for 
 * implementing C++WSP. The rest of the code including all the 
 * modifications to the files derived from these two packages is 
 * copyrighted by Infosys Technologies Ltd.  
 */

#ifndef SERVLET_OUTPUT_STREAM_H
#define SERVLET_OUTPUT_STREAM_H

#include <sstream>
#include "ServletException.h"

#include "../cpp_servlet_dll.h"

// Again inheriting from stringbuf class for lack of finding a 
// good implemented stream class in C++.

class ServletResponse;

class CPP_SERVLET_DLL_API ServletOutputStream  : public std::stringbuf
{


	protected:

		// number of bytes in the stream before you flush it 
		// out
		int _bufferSize;
		int _charCounter;

		// boolean which tells if the headers have been sent
		bool _headersSent;

		// keeping a reference of ServletResponse to call flushBuffer
		ServletResponse *_refServletResp;

	// protected:
	// making it public for now
	public:
		ServletOutputStream( ):_bufferSize(500), _charCounter(0),
			_headersSent(false), _refServletResp(NULL)
		{ }


	void clear()
	{ str(""); _charCounter = 0; }

	void setServletResponseRef (ServletResponse * servRespPtr)
	{ _refServletResp = servRespPtr; }

	bool getHeadersSent()
	{ return _headersSent; }

	void setHeadersSent(bool b=true)
	{ _headersSent = b; }

    
    public:

		void setBufferSize(int bufSize)
		{ _bufferSize = bufSize; }

		int getBufferSize()
		{ return _bufferSize; }

		// This method (flush) should be used by the implementation
		// to read the stream contents and empty the stream
		virtual void flush() = 0;

		// check if the buffer has grown big enough
		// for us to flush it out
		void checkFlushRequired();

		virtual void print(const char * str) throw( ServletException)
		{
			int len = strlen(str);
			if(len <= 0)
				return;

			sputn(str, len);
			_charCounter += len;
			checkFlushRequired();
		}

		virtual void print(const char * str, int len) throw( ServletException)
		{
			if(len <= 0)
				return;

			sputn(str, len);
			_charCounter += len;
			checkFlushRequired();
		}

		virtual void print(std::string s) throw( ServletException)
		{
		
			if(s == "") return;

			int len = s.length();
			sputn(s.c_str(), len);
			_charCounter += len;
			checkFlushRequired();
    }

		virtual void write (char ch);


     void print(bool b) throw( ServletException) {
	std::string msg;
	if (b) {
	    msg = "true";
	} else {
	    msg = "false";
	}
	print(msg);
    }


     void print(char c) throw( ServletException) {
		 write(c);
    }



	 // ADD CODE to implement the writing of 
	 // int, long, double etc. 
	 // After taking care of figuring out which to write first
	 // SMALL ENDIAN, BIG ENDIAN etc. etc. 
     virtual void print(int i) throw( ServletException) {};
	    

     virtual void print(long l) throw( ServletException)
	 {}
	 

     virtual void print(float f) throw( ServletException) {};


     virtual void print(double d) throw( ServletException) {};
	 

     void println() throw( ServletException) {
		 print(std::string("\r\n"));
    }


     void println(std::string s) throw( ServletException) {
	print(s);
	println();
    }


     void println(bool b) throw( ServletException) {
	print(b);
	println();
    }


     void println(char c) throw( ServletException) {
	print(c);
	println();
    }


     void println(int i) throw( ServletException) {
	print(i);
	println();
    }


     void println(long l) throw( ServletException) {
	print(l);
	println();
    }


     void println(float f) throw( ServletException) {
	print(f);
	println();
    }


     void println(double d) throw( ServletException) {
	print(d);
	println();
    }
};

#endif // SERVLET_OUTPUT_STREAM_H
