/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *
 * The code in this file was originally derived from the 
 * Java source code of The Jakarta Project (Tomcat), 
 * which is copyrighted by The Apache Software Foundation (Their 
 * copyright notice shown below). 
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * The original code may have completely changed while converting
 * to C++. It should be noted that only the packages that contain 
 * the Java Servlet API interfaces, namely javax.servlet and 
 * javax.servlet.http of the Jakarta Project are used for 
 * implementing C++WSP. The rest of the code including all the 
 * modifications to the files derived from these two packages is 
 * copyrighted by Infosys Technologies Ltd.  
 */

#ifndef SERVLET_REQUEST_H
#define SERVLET_REQUEST_H

#pragma warning ( disable : 4251 )

#include <string>
#include <vector>
#include <map>

#include "ServletException.h"
#include "ServletInputStream.h"
#include "RequestDispatcher.h"

#include "../cpp_servlet_dll.h"

typedef std::map<std::string, std::string> parameterMap;

class CPP_SERVLET_DLL_API ServletRequest {


	public:

    
	// Cannot support the return of an object as thers is 
	// no generic base ( and also serializable ) object in C++
	// Hence returning a string for this
	 std::string getAttribute(std::string name);
    

	std::vector<std::string> getAttributeNames();
    

	std::string getCharacterEncoding()
	{ return _characterEncoding; }


     void setCharacterEncoding(std::string env) throw (ServletException);


    int getContentLength()
	{ return _contentLength; }
    
    
	void setContentLength(int conLen)
	{ _contentLength = conLen; }
    

     std::string getContentType()
	 { return _contentType; }

	 void setContentType(const char *contentType)
	 { _contentType = contentType; }
    

     virtual ServletInputStream & getInputStream() throw (ServletException) = 0; 
     
    
     std::string getParameter(std::string name)
	 { return getAttribute(name); }
    
         
	 std::vector<std::string> getParameterNames()
	 { return getAttributeNames(); }
    

	 std::vector<std::string> getParameterValues(std::string name);
 

	// Returns the parameter key-value list (std::map<...> ) by reference
	parameterMap & getParameterMap()
	{ return _paramList; }
    
    
     std::string getProtocol()
	 { return _protocol; }
    
    
     void setProtocol(const char * prot)
	 { _protocol = prot; }

     std::string getScheme()
	 { return _scheme; }
    
	 void setScheme(const char * scheme)
	 { _scheme = scheme; }
    

     std::string getServerName()
	 { return _serverName; }
    
     void setServerName(const char * serverN)
	 { _serverName = serverN; }
    

     int getServerPort()
	 { return _serverPort; }
    
	 void setServerPort(int serverPort)
	 { _serverPort = serverPort; }

    
     std::string getRemoteAddr()
	 { return _remoteAddress; }
    
     void setRemoteAddr(const char * remAddr)
	 { _remoteAddress = remAddr; }
    

     std::string getRemoteHost()
	 { return _remoteHost; }
    
     void setRemoteHost(const char * remoteHost)
	 { _remoteHost = remoteHost; }
    
     void setAttribute(std::string name, std::string  o);
    
    

     void removeAttribute(std::string name);
    
    
	 std::string  getLocale();
    
    
	 std::vector<std::string> getLocales();
    

     bool isSecure()
	 { return _isSecure; } 
    
    
     RequestDispatcher getRequestDispatcher(std::string path);
    
    
	 protected:

		 parameterMap _paramList;

		 bool _isSecure;

		 std::string _characterEncoding;
		 std::string _contentType;
		 int _contentLength;
    
		 std::string _protocol;
		 std::string _scheme; // http/https/ftp

		 std::string _serverName; // web server host name
		 int _serverPort; // web server port name

		 std::string _remoteAddress; // ip address of the client
		 std::string _remoteHost;

		 // Locale values list (If client does not send it in header
		 // then populate with that of server
		 std::vector<std::string> _localeList;

};

#endif // SERVLET_REQUEST_H