/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *
 * The code in this file was originally derived from the 
 * Java source code of The Jakarta Project (Tomcat), 
 * which is copyrighted by The Apache Software Foundation (Their 
 * copyright notice shown below). 
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * The original code may have completely changed while converting
 * to C++. It should be noted that only the packages that contain 
 * the Java Servlet API interfaces, namely javax.servlet and 
 * javax.servlet.http of the Jakarta Project are used for 
 * implementing C++WSP. The rest of the code including all the 
 * modifications to the files derived from these two packages is 
 * copyrighted by Infosys Technologies Ltd.  
 */

#ifndef SERVLET_RESPONSE_H
#define SERVLET_RESPONSE_H
 
#include <string>
#include <vector>
#include <map>

#include "ServletException.h"
#include "ServletOutputStream.h"
#include "RequestDispatcher.h"

#include "../cpp_servlet_dll.h"

class CPP_SERVLET_DLL_API ServletResponse {



	protected:
		std::string _characterEncoding;
		ServletOutputStream *_servletOpStream;
		std::string _contentType;
		std::string _locale;
		int _contentLength;
		int _bufferSize;
		bool _responseCommited;

	public:
    

		ServletResponse():_responseCommited(false),
			_servletOpStream(NULL), _bufferSize(500)
		{}


	std::string getCharacterEncoding()
	{ return _characterEncoding; }
    
    
	std::string getContentType()
	{ return _contentType; }


	 void setOutputStream(ServletOutputStream * outStr)
	 {
		 _servletOpStream = outStr;
		 _servletOpStream->setBufferSize(_bufferSize);
		 _servletOpStream->setServletResponseRef(this);
	 }

     ServletOutputStream & getOutputStream() throw( ServletException);
	 
     virtual void setContentLength(int len) = 0;
    
     virtual void setContentType(std::string type) = 0;
    

     void setBufferSize(int size)
	 { _bufferSize = size; }
    

     int getBufferSize()
	 { return _bufferSize; }
    

     virtual void flushBuffer() throw(ServletException) = 0;
    

     virtual void resetBuffer() throw(ServletException) = 0;
    
     bool isCommitted()
	 { return _responseCommited; }
    
     virtual void reset() throw(ServletException) = 0;
    
     virtual void setLocale(std::string loc) = 0;
    
	 std::string getLocale()
	 { return _locale; }



};


#endif // SERVLET_RESPONSE_H

