/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#pragma warning ( disable : 4786 )

#include "ApacheHttpServletResponse.h"

extern "C"
{
#include "http_protocol.h"
#include "http_core.h"
}



#define LOCATION_HEADER_KEY "Location" 



void ApacheHttpServletResponse::sendHeaders() throw(ServletException)
{
	
	// Need to call this base class protected 
	// method to set the Set-Cookie header
	addCookiesToHeaders();

	std::string contentType = getContentType();
	// e.g. of contentType = text/html
	ap_set_content_type(_r, contentType.c_str());

	// iterate through the header list and send other headers
	
	// DON'T USE MAP HERE DIRECTLY 
	// servRespHeaderMap::iterator mapIter = _servRespHdrList.begin();
	initializeMapIter();

	std::string key, val;
	// while (mapIter !=  _servRespHdrList.end() )
	while ( 0 == getNextHeader(key, val) )
	{
		// Add the header entries to the headers_out table
		//key = (*mapIter).first;
		//val = (*mapIter).second;
		
		// apr_table_set(_r->headers_out, key.c_str(), val.c_str() );
		// The one above (apr_table_set) will over write the entries
		apr_table_add(_r->headers_out, key.c_str(), val.c_str() );

		//mapIter++;

		// Could have also used the apr_array_make/ apr_array_push calls
		// started but commented out below
		// apr_table_entry_t *hdr_entry = (apr_table_entry *) apr_array_push(_r->headers_out.a);
		
		// APR_DECLARE(apr_array_header_t *) apr_array_make(apr_pool_t *p,
        //                                       int nelts, int elt_size);

		// APR_DECLARE(void *) apr_array_push(apr_array_header_t *arr);

		// paramNames.push_back((*mapIter).first );
		
	}
	
	_headersSent=true;

  
}

void ApacheHttpServletResponse::sendErrorHeaders()
{
	
	// Need to call this base class protected 
	// method to set the Set-Cookie header
	addCookiesToHeaders();

	//std::string contentType = getContentType();

	// e.g. of contentType = text/html
	// ap_set_content_type(_r, contentType.c_str());

	// iterate through the header list and send other headers

	// DON'T USE MAP HERE DIRECTLY - AS STL DLL COMBINATION 
	// CAUSES PAIN !!!
	
	initializeMapIter();

	std::string key, val;
	// while (mapIter !=  _servRespHdrList.end() )
	while ( 0 == getNextHeader(key, val) )
	{
		apr_table_add(_r->err_headers_out, key.c_str(), val.c_str() );
	}
	_headersSent=true;
}


void ApacheHttpServletResponse::sendError(int sc, std::string msg) throw( ServletException )
{
	if(_responseCommited)
	{
		ServletException servExcep("Response already commited");
		throw servExcep;
	}

	_requestStatus = sc;
	_responseCommited = true;

	// Ignoring the message msg for now. 
	// ADD CODE to handle the message appropriately after 
	// developing more understanding about that.

	//sendHeaders();
	clear();
	_r->status = _requestStatus;

	sendErrorHeaders();
	// Don't use this --> ap_send_error_response(_r, SC_OK);
	// Use like below (pass 0) 
	ap_send_error_response(_r, 0);
}


void ApacheHttpServletResponse::sendError(int sc) throw( ServletException )
{
	if(_responseCommited)
	{
		ServletException servExcep("Response already commited");
		throw servExcep;
	}

	_requestStatus = sc;
	_responseCommited = true;

	//sendHeaders();
	clear();
	_r->status = _requestStatus;
	sendErrorHeaders();
	ap_send_error_response(_r, 0);
	

}

void ApacheHttpServletResponse::sendRedirect(std::string location) throw (ServletException)
{
	if(_responseCommited)
	{
		ServletException servExcep("Response already commited");
		throw servExcep;
	}

	
	std::string redirectURI = location;
	// ADD CODE to compute the absolute URL from the relative 
	// one - see the comments in the header file


	_requestStatus = SC_MOVED_TEMPORARILY;
	_responseCommited = true;

	_r->status = _requestStatus;
	apr_table_add(_r->headers_out, LOCATION_HEADER_KEY, redirectURI.c_str() );

	sendHeaders();

	clear();

	//	ap_custom_response(_r, _requestStatus, redirectURI.c_str() );
	//ap_send_error_response(_r, 404);
	

}


// Not very clear of the use of this method 
// Check the implementation after finding out the use - Khushnood
void ApacheHttpServletResponse::setStatus(int sc)
{
	_requestStatus = sc;

	// The container clears the buffer and sets the Location header, preserving
     // * cookies and other headers.

	// Don;t know why do we need to add Location headers

	// How do we take care of the above ???
	_r->status = sc;
	sendHeaders();
	
	clear();

	// ap_custom_response(_r, _requestStatus, "" );
	
	

}