/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


// This file contains the definition of the handler function or the module.
// Since it utilizes C++ libraries it is written separately as C++ code, 
// but under extern "C" so that it is able to be called as an apache hook 
// fn. which are in C

#pragma warning ( disable : 4786 )

// ************** NOTE  __ IMPORTANT *****************
// If _DLL is defined and run time library is "debug multi-threaded
// dll then HttpServletResponse.h (basically headers including template classes)
// should be included before the apache's stuff e.g. httpd.h
// ************** NOTE  __ IMPORTANT *****************

// Commenting these two for now - as need to 
// focus on the In-process development
// #include "cpp_servlet_proxy.h"
// #include "tryHttpServletRequest.h"


#include <sstream>
#include "base_servlet/ServletInputStream.h"
#include "servlet_container.h"
#include "HttpServletRequest.h"
#include "ApacheHttpServletResponse.h"
#include "ApacheServletOutputStream.h"
// #include "user_servlets_dll.h"
#undef pid_t
#include "MessageLogger.h"

extern "C"
{

#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_main.h"
#include "http_protocol.h"
#include "http_request.h"
#include "util_script.h"
#include "http_connection.h"

// #include "apr_strings.h"

#include <stdio.h>

// functions and data declared/defined in mod_try.c
#include "mod_try.h"




// referring to all the data in mod_try.c 
// which were earlier defined as static. Now made them non static 
// there and referring to them by extern directive 

//extern const char *trace;
//extern apr_table_t *static_calls_made;
//extern apr_pool_t *cpp_servlet_pool;
//extern apr_pool_t *cpp_servlet_subpool;
}


// To avoid clash between ACE and Apache's typedef of pid_t
#undef pid_t


extern "C"
{

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Now we declare our content handlers, which are invoked when the server   */
/* encounters a document which our module is supposed to have a chance to   */
/* see.  (See mod_mime's SetHandler and AddHandler directives, and the      */
/* mod_info and mod_status examples, for more details.)                     */
/*                                                                          */
/* Since content handlers are dumping data directly into the connexion      */
/* (using the r*() routines, such as rputs() and rprintf()) without         */
/* intervention by other parts of the server, they need to make             */
/* sure any accumulated HTTP headers are sent first.  This is done by       */
/* calling send_http_header().  Otherwise, no header will be sent at all,   */
/* and the output sent to the client will actually be HTTP-uncompliant.     */
/*--------------------------------------------------------------------------*/
/* 
 * Sample content handler.  All this does is display the call list that has
 * been built up so far.
 *
 * The return value instructs the caller concerning what happened and what to
 * do next:
 *  OK ("we did our thing")
 *  DECLINED ("this isn't something with which we want to get involved")
 *  HTTP_mumble ("an error status should be reported")
 */

#define MSG_BODY_BUF_SIZE 2048

int iFlag=0;

/**
NOT USED NOW AS ACE THREADS ARE WORKING FINE
static void * APR_THREAD_FUNC startTimerThread(apr_thread_t *thd, void *dummy)
{
	// iFlag=0;
	// request_rec *r = (request_rec*)dummy;
	// ap_rputs("apr_thread_create: In Thread function",r);
	// ap_rflush(r);
	ServletContainer * sessCtr = (ServletContainer*)dummy;
	sessCtr->sessionCleanupFunction();
	return 0;
}

int createCPPServletTimerThread(request_rec *r, ServletContainer *sessCtr)
{
	apr_status_t rv;
    apr_threadattr_t *thread_attr;
    apr_thread_t *start_thread_id;

	
	apr_threadattr_create(&thread_attr, cpp_servlet_pool);
    // 0 means PTHREAD_CREATE_JOINABLE 
    apr_threadattr_detach_set(thread_attr, 0);

    rv = apr_thread_create(&start_thread_id, thread_attr, startTimerThread,
                           (void*)sessCtr, cpp_servlet_pool);
    if (rv != APR_SUCCESS) {
        ap_rputs("apr_thread_create: unable to create worker thread",r);
		return -1;
    }
	else
	{
        ap_rputs("apr_thread_create: SUCCESS",r);

	}
	return 0;
}

**/



ServletContainer *servletContPtr=NULL;

int cpp_servlet_handler(request_rec *r)
{

	// Dummy call to load user servlets
	// nUser_servlets_dll = 1;

	if(servletContPtr == NULL)
	{

		std::string webRoot(ap_server_root);
		webRoot.append("/cpp_servlet/");

		std::string logFile(ap_server_root);
		logFile.append("/logs/cpp_servlet_log.txt");
		MessageLogger::setLogFile((char*)logFile.c_str());
		MessageLogger::setLogLevel(eLogDebug);
		
		MessageLogger::LogMessage('t', "try_handler", "Before ServletContainer creation ");
	
		servletContPtr = new ServletContainer(webRoot);
		
		MessageLogger::LogMessage('t', "try_handler", "After ServletContainer creation ");

	}

	
	MessageLogger::LogMessage('t', "try_handler", "Start cpp_servlet_handler");
	
	
	// MessageLogger::LogMessage('D', "try_handler", "place: \"%s\"", "In try handler");

	// MessageLogger::LogMessage('d', "try_handler", "Http root = %s", ap_server_root);

    // cpp_servlet_cfg *dcfg;

    if (strcmp(r->handler, "cpp_servlet")) {
        return DECLINED;
    }


    // dcfg = our_dconfig(r);
 

/*
	if(servletContPtr == NULL)
	{
		MessageLogger::LogMessage('t', "try_handler", "Before ServletContainer creation ");
	
		servletContPtr = new ServletContainer(webRoot);
		
		MessageLogger::LogMessage('t', "try_handler", "After ServletContainer creation ");

	}
*/
  
	if (iFlag == 0)
	{
		iFlag = 1;
		// createCPPServletTimerThread(r, servletContPtr);
	}

	/***
	   Adding the call to ACE server here 
	   ***/

	
	/** Get the body of the message - if any **/

	/** set up the client to send body data **/
	if (OK != ap_setup_client_block(r, REQUEST_CHUNKED_DECHUNK) )
	{
		ap_rputs("Could not setup client properly to receive body contents <BR> /n", r);
		MessageLogger::LogMessage('e', "try_handler", "Could not setup client properly to receive body contents");
	}

	HttpServletRequest httpReqObj;
 
    
	int hasBody = ap_should_client_block(r);
	char msgBody[MSG_BODY_BUF_SIZE];

	// Creating a string based buffer
	// std::stringbuf msgBodyStrBuf;
	// Using a ServletInputStream class which derives from std::stringbuf 
	ServletInputStream & msgBodyStrBuf = httpReqObj.getInputStream();

	// std::string msgBodyStr;
	// msgBodyStr
	if(hasBody)
	{
		// ap_rputs("Reading the body of the message <BR> \n", r);
		int readStatus = 0;
		while ( (readStatus = ap_get_client_block( r, msgBody, MSG_BODY_BUF_SIZE) ) > 0 )
		{
			// msgBody[readStatus] = 0; // is it nedded (putting eostr in the end) ? 
			// msgBodyStr += msgBody;
			msgBodyStrBuf.sputn(msgBody, readStatus );
		}
		if(readStatus == -1)
		{
			ap_rputs("Error reading request message body <BR> /n", r);
			MessageLogger::LogMessage('e', "try_handler", "Error reading request message body");
		}
		// Not printing , msgBodyStrBuf.str().c_str()  contents 
		// as it is a stream once read you can't read again later
		//ap_rprintf(r, "The request contains a message body. ");
	}
	else
	{
		//ap_rputs(" The request does not contain a body <BR>\n", r);
	}

	/**
	if( r->read_body == REQUEST_NO_BODY )
	{
			ap_rputs(" The request does not contain a body <BR>\n", r);
	}
	else if ( r->read_body == REQUEST_CHUNKED_ERROR )
	{
		ap_rputs(" Content Length a must: Send 411 error if body without Content-Length <BR>\n", r);
	}
	else if ( r-> read_body == REQUEST_CHUNKED_DECHUNK )
	{
		ap_rputs(" If chunked, I'll remove the chunks for the caller <BR>\n", r);
	}
	**/

	/**
	Commenting this out to be used with outh-of-proc servlet 
	container .

	CPPServletProxy myCppServletprxy;
	TryHttpServletRequest myCppReqStruct;
	myCppReqStruct.reqLine (r->the_request);
	if(r->args != NULL)
		myCppReqStruct.queryArgs( r->args);
	
	std::string respStr;
	
	if(myCppServletprxy.isValid() )
	{
		if(0 != myCppServletprxy.forwardRequest(myCppReqStruct, respStr))
		{
			ap_rprintf( r, "%s", myCppServletprxy.getErrMsg().c_str() );
		}
	}
	else
	{
		ap_rprintf( r, "%s", myCppServletprxy.getErrMsg().c_str() );
	}
	***/

	
	std::string respStr;
	
	httpReqObj.setRequestLine(r->the_request);
	if(r->args)
		httpReqObj.setQueryParams(r->args);

	httpReqObj.setMethod(r->method);

	if(r->user)
		httpReqObj.setUser(r->user);

	if(r->ap_auth_type)
		httpReqObj.setAuthType(r->ap_auth_type);
	
	if(r->content_encoding)
		httpReqObj.setCharacterEncoding(r->content_encoding);

	httpReqObj.setContentLength(r->clength);

	if(r->protocol)
		httpReqObj.setProtocol(r->protocol);

	if(r->content_type)
		httpReqObj.setContentType(r->content_type);

	
	// data got from server struct
	if(r->server && r->server->server_hostname)
		httpReqObj.setServerName(r->server->server_hostname);

	if(r->server && r->server->port)
		httpReqObj.setServerPort(r->server->port);

	
	// data got from the connection struct
	
	if( r->connection && r->connection->remote_addr)
	{
		char *clientIpaddr;
		apr_sockaddr_ip_get (  &clientIpaddr, 
			( apr_sockaddr_t * ) r->connection->remote_addr );
      
		httpReqObj.setRemoteAddr(clientIpaddr);

		if(r->connection->remote_addr->servname)
			httpReqObj.setRemoteHost( r->connection->remote_addr->servname);
	}

	
	// uri specfic data and also  from the uri struct -- r->parsed_uri->
	if(r->parsed_uri.scheme)
		httpReqObj.setScheme(r->parsed_uri.scheme);

	if(r->path_info)
		httpReqObj.setPathInfo(r->path_info);
	// will be done by servlet engine httpReqObj.setPathTranslated()

	if(r->unparsed_uri)
		httpReqObj.setRequestURI(r->unparsed_uri);

	if(r->uri)
		httpReqObj.setRequestURL(r->uri);


	// Add all the headers to the request
	
	// apr_array_header_t *hdrs_arr = &(r->headers_in->a); // (apr_array_header_t *) apr_table_elts(r->headers_in);
	const apr_array_header_t *hdrs_arr = apr_table_elts(r->headers_in); // (apr_array_header_t *) apr_table_elts(r->headers_in);

	apr_table_entry_t *hdr_entry = (apr_table_entry_t *)hdrs_arr->elts;
    //int i;
  
    for (int i = 0; i < hdrs_arr->nalloc; i++) 
	{
		 // Add it to the req obj
		 httpReqObj.addHeader(hdr_entry[i].key,
			hdr_entry[i].val);
	}

	// Construct the ApacheHttpServletResponse object
	ApacheHttpServletResponse *httpSrvltRespObj = new ApacheHttpServletResponse(r);

	// Construct the ApacheServletOutputStream object 
	// and set it on the servlet response obj. 
	ApacheServletOutputStream *apchSvltOpStrObj = new ApacheServletOutputStream;
	apchSvltOpStrObj->setApacheRequestStructure(r);
	httpSrvltRespObj->setOutputStream(apchSvltOpStrObj);

	try
	{
		MessageLogger::LogMessage('t', "try_handler", "Before servicing the request ");

		servletContPtr->serviceRequest(httpReqObj, *httpSrvltRespObj);

		MessageLogger::LogMessage('t', "try_handler", "After servicing the request ");
	}
	catch(ServletException e)
	{
		ap_rprintf(r, "Caught Exception : %s ", e.getRootCause().c_str() );
		MessageLogger::LogMessage('e', "try_handler", "Caught Exception : %s", e.getRootCause().c_str());
	}

	
	// BEGIN DEBUG MESSAGES
	
	//MessageLogger::LogMessage('D', "try_handler", "Protocol: \"%s\"", httpReqObj.getProtocol().c_str());
	//MessageLogger::LogMessage('D', "try_handler", "Content Type: \"%s\"", httpReqObj.getContentType().c_str());
	//MessageLogger::LogMessage('d', "try_handler", "Content encoding: \"%s\"", httpReqObj.getCharacterEncoding().c_str());
	//MessageLogger::LogMessage('d', "try_handler", "Content Length: \"%d\"", httpReqObj.getContentLength());
	//MessageLogger::LogMessage('d', "try_handler", "Client IP address: \"%s\"", httpReqObj.getRemoteAddr().c_str());
	//MessageLogger::LogMessage('d', "try_handler", "Unparsed URI: \"%s\"", httpReqObj.getRequestURI().c_str());
	//MessageLogger::LogMessage('d', "try_handler", "URL: \"%s\"", httpReqObj.getRequestURL().c_str());
	//MessageLogger::LogMessage('d', "try_handler", "Path Info: \"%s\"", httpReqObj.getPathInfo().c_str());
	//MessageLogger::LogMessage('D', "try_handler", "Printing context path after calling service: \"%s\"", httpReqObj.getContextPath().c_str());
	//MessageLogger::LogMessage('D', "try_handler", "The header fields from the browser");
	//for (i = 0; i < hdrs_arr->nalloc; i++) 
	//{
		 //MessageLogger::LogMessage('d', "try_handler", "name: \"%s\" ; value: \"%s\"",
										// hdr_entry[i].key, hdr_entry[i].val);
	//}
	// END DEBUG MESSAGES
	

	ap_rflush(r);
	
	delete httpSrvltRespObj;
	delete apchSvltOpStrObj;

	//char tryStr[50];
	//strcpy(tryStr, "<br> <B> Hello from ap_rwrite method </B> \n");
	//ap_rwrite(tryStr, strlen(tryStr), r);
	//ap_rflush(r);

	MessageLogger::LogMessage('t', "try_handler", "End cpp_servlet_handler");

	/*
     * We did what we wanted to do, so tell the rest of the server we
     * succeeded.
     */
    return OK;
}


}