/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#pragma warning ( disable : 4786 )

#include "DeploymentConfigManager.h"
#include "ApplicationContextManager.h"

#include "ace/Dirent.h"
#include "ace/OS.h"

#include "MessageLogger.h"

void DeploymentConfigManager::initialize(const char *rootPath)
{

	ServletContext *srvContext;
	ServletConfigDictionary *srvConfigDict;
	// std::string dllName;

	ACE_Dirent dir (ACE_TEXT (rootPath));
	char buff[200];
	ACE_stat fileStat;
	dirent *directory;

	while ((directory = dir.read ()) != 0 )
	{
		memset(buff,0,200);
		strcat(buff, rootPath);
		strcat(buff, directory->d_name);
		
		ACE_OS::stat(buff, &fileStat);

		if (fileStat.st_mode & _S_IFDIR)
		{
			strcat(buff, "/web.xml");
			ApplicationContextManager appContextMgr(buff);
			srvContext = appContextMgr.getServletContext();
			srvConfigDict = appContextMgr.getServletConfigDictionary();
			// dllName = appContextMgr.getDeploymentDLLName();

			_srvContextMap [directory->d_name] = srvContext;
			_srvConfigDictMap [directory->d_name] = srvConfigDict;
			// _srvDLLMap [directory->d_name] = dllName;
		}
	}

}

ServletContext* DeploymentConfigManager::getServletContext(std::string appName)
{
	servletContextMap::iterator iterator;
	
	iterator = _srvContextMap.find(appName);
	
	if (iterator != _srvContextMap.end())
	{
		return (*iterator).second;
	}
	else
	{
		return NULL;
	}		
}

ServletConfig* DeploymentConfigManager::getServletConfig(std::string appName, std::string servletName)
{
	servletConfigDictionaryMap::iterator iterator;

	iterator = _srvConfigDictMap.find(appName);
	if (iterator != _srvConfigDictMap.end())
	{
		ServletConfigDictionary *srvConfigDictionary = (*iterator).second;
		return srvConfigDictionary->getServletConfig(servletName);
	}
	else
	{
		return NULL;
	}
}

/*
std::string DeploymentConfigManager::getDeploymentDLLName(std::string appName)
{
	ServletContext *context = getServletContext(appName);
	if (context != NULL)
	{
		return context->getDLLName();
	}
	else
	{
		return "";
	}
}
*/

std::vector<std::string> DeploymentConfigManager::getAllDeploymentDLLNames()
{
	std::vector<std::string>		vDllNames;
	std::string						dllName;
	ServletContext					*context;

	servletContextMap::iterator iterator;
	
	for(iterator = _srvContextMap.begin(); iterator != _srvContextMap.end(); iterator++)
	{
		context = (*iterator).second;
		
		if (context != NULL)
			dllName = context->getDLLName();
		else
			continue;

		vDllNames.push_back(dllName);
	}

	return vDllNames;
}