/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#pragma warning ( disable : 4786 )

#include "httpRequestTextParser.h"

#define QUERY_PARAM_SEP "&"
#define QUERY_VALUE_SEP "="

int HttpRequestTextParser::parseAndFillParams(HttpServletRequest & httpReq)
{
	// ADD code
	std::string queryParams = httpReq.getQueryString();
	
	// First parse the query parameters to create a 
	// parameter list
	parseKeyValueString(queryParams, httpReq);
	

	// Now parse for parameters which have been posted in the 
	// body of the message
	ServletInputStream & servletIpStr = httpReq.getInputStream();
	
	// This may be expensive if the body is big
	// if thats the case then this could be improvised 
	// by just reading char(s) in bunches and deriving key -
	// value pairs. For now keeping things simple for coding. - K. Naqvi
	std::string bodyStr = servletIpStr.str();
	//	while( (ch = servletIpStr.sbumpc() ) != EOF)
	
	parseKeyValueString(bodyStr, httpReq);

	return 0;
}

void HttpRequestTextParser::parseKeyValueString(std::string &paramLine, HttpServletRequest & httpReq)
{
	int paramSepPos = 0, valueSepPos = 0, currentCount = 0;
	std::string paramKey, paramValue;

	// TILL we fix this allow looping only for 5 parameres
	// Provide logging (trace ) to catch bugs
	int bugLoopCount = 0;

	do
	{
		if( (valueSepPos = paramLine.find_first_of(QUERY_VALUE_SEP, paramSepPos) ) == std::string::npos)
			break;

		if(paramSepPos != 0)
			paramSepPos += 1; // increment by 1 for anything after the first param

		paramKey = paramLine.substr(paramSepPos , (valueSepPos - paramSepPos ) );

		if( (paramSepPos = paramLine.find_first_of(QUERY_PARAM_SEP, valueSepPos) ) != std::string::npos)
		{
			paramValue = paramLine.substr(valueSepPos + 1, (paramSepPos - valueSepPos - 1) );
		}
		else
		{
			// read till the end and break from loop
			paramValue = paramLine.substr(valueSepPos + 1);
		}
		remove_escape_std_str(paramKey);
		remove_escape_std_str(paramValue);
		httpReq.setAttribute(paramKey, paramValue);
		// &&  ++bugLoopCount <= 1 
	} while( paramSepPos != std::string::npos ); // exit loop  on breaks within also
}

void HttpRequestTextParser::remove_escape(char * str)
{
  char * from = str;
  char * to   = str;
  while ( *from != '\0' )
  {
    char ch = *from++;
    switch ( ch )
    {
      case '%' :
        ch = (hex(*from++)<<4) | hex(*from++);
        break;
      case '+' :
        ch = ' '; break;
    }
    *to++ = ch;
  }
  *to = '\0';
}

void HttpRequestTextParser::remove_escape_std_str(std::string & str)
{
	int j=0;
	for(int i=0; i< str.length(); i++)
	{
		char ch = str[i];
		switch ( ch )
		{
		case '%' :
			{
				ch = (hex(str[++i])<<4) | hex(str[++i]);
				break;
			}
		case '+' :
			{
				ch = ' '; 
				break;
			}
		}
		str[j++] = ch;
	}
	// str[j] = '\0';
	str.resize(j);
}

int HttpRequestTextParser::hex( char c )
{
  // No check is done for values greater than f (assumed it will be used 
  // only for http request posted data)
  if ( isdigit( c ) )
    return c-'0';
  if ( isalpha( c ) )
    return tolower(c)-'a'+10; 
  return 0;
}
