/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#pragma warning ( disable : 4786 )

#define CPP_SERVLET_IN_URL "cpp_servlet"

#define CONTENT_TYPE_HDR_KEY "Content-Type"
#define DEFAULT_CONTENT_TYPE "application/x-www-form-urlencoded" 
#define MULTIPART_CONTENT_TYPE "multipart/form-data"

#include "session_mgr.h"
#include "servlet_container.h"
#include "httpRequestTextParser.h"
#include "httpRequestMultipartParser.h"
#include "ServletFactory.h"
#include "HttpServletResponse.h"
#include "share_factory_instance.h"
#include "MessageLogger.h"

HttpServlet * f1()
{
	return new HttpServlet(); // dummy
}

ServletContainer::ServletContainer(std::string webRoot):_initialized(false), _webRoot(webRoot)
{
	_sessionMgr = new SessionManager();
	initialize();
}

ServletContainer::~ServletContainer()
{
	delete _sessionMgr;
	delete _dplyConfigMgr;
}

int ServletContainer::serviceRequest(HttpServletRequest & httpReq,
					 HttpServletResponse & resp)
{

	MessageLogger::LogMessage('t', "servletContainer", "Start serviceRequest");
	// ADD CODE
	// Find out the "Content-Type" from the header
	std::string contentType = httpReq.getHeader(CONTENT_TYPE_HDR_KEY);
	
	// ADD CODE To construct a Multipart parser if contentType is
	// multipart

	HttpRequestTextParser httpTxtParser;
	// since every thing is within the request itself 
	// don't have to pass any thing specific
	httpTxtParser.parseAndFillParams(httpReq);

	std::string reqSessId = httpReq.getRequestedSessionId();
	HttpSession * sessObj = NULL;
	if(reqSessId != "")
	{
		// call the session manager to get the session obj
		// set it to req obj
		
		MessageLogger::LogMessage('t', "servletContainer", "Before getting session object");
		
		sessObj = _sessionMgr->getSessionObj(reqSessId);

		MessageLogger::LogMessage('t', "servletContainer", "After getting session object");
		
		if(sessObj != NULL)
			httpReq.setHttpSessionObj(sessObj);
	}

	if(sessObj == NULL)
	{
		// call the session manager to generate session id
		// Set it to req obj
		MessageLogger::LogMessage('T', "servletContainer", "Before generating session id");
		
		reqSessId = _sessionMgr->generateSessionId();
		
		MessageLogger::LogMessage('T', "servletContainer", "After generating session id");
		
		httpReq.setSessionId(reqSessId.c_str() );
	}

	std::string urlStr, cppServletName, webAppName;
	urlStr = httpReq.getRequestURL();
	int befLastSlash=0, prevSlash=0, appNameSlash=0;
	befLastSlash = urlStr.find_last_not_of("/\\");
	// to take care of multiple slashes occuring
	// befLastSlash = urlStr.find_last_not_of("/\\", befLastSlash);

	if(befLastSlash == std::string::npos)
	{
		// throw exception
		ServletException svltExcp(std::string("Illegal URL encountered " ) );
		throw svltExcp;
	}

	prevSlash = urlStr.find_last_of("/\\", befLastSlash - 1);

	if(prevSlash == std::string::npos)
	{
		// throw exception
		ServletException svltExcp(std::string("Illegal URL encountered " ) );
		throw svltExcp;
	}

	cppServletName = urlStr.substr(prevSlash + 1, ( befLastSlash - prevSlash) );

	if (cppServletName == CPP_SERVLET_IN_URL)
	{
		// throw exception
		ServletException svltExcp(std::string("web-app name/servlet name NOT entered " ) );
		throw svltExcp;
	}

//Yassar
	
	// Get the app-name here

	appNameSlash = urlStr.find_last_of("/\\", prevSlash - 1);

	if(appNameSlash == std::string::npos)
	{
		//throw exception
		ServletException svltExcp(std::string("Illegal URL encountered "));
		throw svltExcp;
	}

	webAppName = urlStr.substr(appNameSlash + 1, (prevSlash - (appNameSlash+1)));
	
	//MessageLogger::LogMessage('D', "ServletContainer", "WebApp Name is %s", webAppName.c_str());

	if (webAppName == CPP_SERVLET_IN_URL)
	{
		// throw exception
		ServletException svltExcp(std::string("web-app name/servlet name NOT entered " ) );
		throw svltExcp;
	}

// Yassanr - end

	// set the request context path
	// As of now setting it to "/"
	// When bring in app contexts in picture then will 
	// set it till "/app_context"
	//std::string ctxPath = httpReq.getPathInfo();
	// get the Path Infp till app-context (i.e. before servlet name)
	
// std::string ctxPath = std::string("/") + CPP_SERVLET_IN_URL + "/";

// Yassar - should replace the abouve line by this
	
	 std::string ctxPath = std::string("/") + CPP_SERVLET_IN_URL + "/" + webAppName;
	 // std::string dllName = _dplyConfigMgr->getDeploymentDLLName(webAppName);

 	// ctxPath += "/";
	httpReq.setContextPath(ctxPath.c_str() );
	// httpReq.setContextPath("");

	// ServletFactory::registerToFactory(cppServletName.c_str(), &f1);

	// This call is to make sure that the two static variables,
	// one in this module and another in the user servlets dll, which 
	// hold the ServletFactory instance are the same. 
	// Since all the servlets would register to the ServletFactory instance 
	// in the user_servlet dll making that instance pointer as mine too
	// ServletFactory::setInstance(get_user_servlet_lib_factory());


// Moving this part (Loading DLL) to the initialize function.
/*
#ifdef WIN32
	// Instead of the above call can call LoadLibrary 
	// atleast on Win32 need to look for a similar function 
	// on unix
	
	// should be doing this instead of the hardcoded dll name
	
	// Construct the complete path here using _webRoot and dllName
	std::string dllPath;
	dllPath.append(_webRoot);
	dllPath.append("../modules/");
	dllPath.append(dllName);
	MessageLogger::LogMessage('D', "ServletContainer", "DLL Name is %s", dllName.c_str());
	MessageLogger::LogMessage('D', "ServletContainer", "DLL Path is %s", dllPath.c_str());
	LoadLibrary(dllPath.c_str());

#else
	// Need to call the appropriate function on unix
	// Just a dummy call to load the user_servlet_dll
	get_user_servlet_lib_factory();
#endif
*/
	//MessageLogger::LogMessage('D', "ServletContainer", "Get the servlet from dll: \"%d\"", (int) ServletFactory::instance());

	// HttpServlet * servletObj = f1();
	// HttpServlet *servletObj = new TryExampleServlet();

	MessageLogger::LogMessage('t', "servletContainer", "Before getting servlet");

	HttpServlet * servletObj = ServletFactory::getServlet(cppServletName.c_str());

	MessageLogger::LogMessage('T', "servletContainer", "After getting servlet");
	
	// HttpServlet * servletObj =  get_servlet_from_dll_factory(cppServletName.c_str() );
	
	if( servletObj == NULL)
	{
		MessageLogger::LogMessage('E', "ServletContainer", "servlet Factory could not create servlet : \"%s\"", cppServletName.c_str() );
		ServletException svltExcp(std::string("Servlet Factory could not create servlet : " + cppServletName) );
		throw svltExcp;
	}

	// Set the response object reference in the request object
	httpReq.setHttpServletResponseObjRef(&resp);

	//MessageLogger::LogMessage('D', "try_handler", "Got the servlet  \"%s\"", cppServletName.c_str() );

	// Get the config object from the deploymentManager passing appName and servletName

//Yassar

	MessageLogger::LogMessage('t', "servletContainer", "Before getting servletConfig ");
	
	ServletConfig* config = _dplyConfigMgr->getServletConfig(webAppName, cppServletName);

	MessageLogger::LogMessage('T', "servletContainer", "After getting servletConfig");
	
	
	if( config == NULL)
	{
		MessageLogger::LogMessage('E', "ServletContainer", "servlet Factory could not get servlet Config: \"%s\"", cppServletName.c_str() );
		ServletException svltExcp(std::string("Servlet Factory could not get Config data for the servlet " + cppServletName + "\nCheck the URL typed") );
		throw svltExcp;
	}

	// std::string bgColour = config->getInitParameter("colour");
	// MessageLogger::LogMessage('D', "servletcontainer", "BG colour from container is %s", bgColour.c_str() );

	// Here servletObj->init() shoulb be called passing the config object for the servlet.
	servletObj->init(*config);

	//Yassar end

	MessageLogger::LogMessage('t', "servletContainer", "Before calling servlet->service");

	// Invoke the servlet's service method
	servletObj->service( httpReq, resp);
	// job is done so delete it after calling the destroy method on it

	MessageLogger::LogMessage('t', "servletContainer", "After calling servlet->service");

	servletObj->destroy();

	delete servletObj;

	//MessageLogger::LogMessage('D', "try_handler", "After calling the service method \"%s\"", "");

	// debug msg
	ServletOutputStream & srvltOpStream = resp.getOutputStream();
	// srvltOpStream.print("after service method <br>\n");
	srvltOpStream.flush();

	// ADD CODE to check it the servlet started a session
	// If so save the session using Yassar's Session mgr

	if(httpReq.newSessionCreated() )
	{
		// add the session obj to the session manager
		HttpSession & sessObj = httpReq.getSession();
		srvltOpStream.print("new session created <br>\n");
		srvltOpStream.flush();
		
		// The session Object is allocated on the Heap (new is done)
		// inside the request obj.
		// The session mgr should free it when it times out
		_sessionMgr->addSessionObj(&sessObj);

	}

	// srvltOpStream.print("after doing everything <br>\n");
	srvltOpStream.flush();
	MessageLogger::LogMessage('T', "servletContainer", "End serviceRequest");

	return 0;
}

void ServletContainer::initialize()
{
	MessageLogger::LogMessage('t', "servletContainer", "Start Servlet Container initialize");
	if(!_initialized)
	{
		std::vector<std::string> vDllNames;
		std::vector<std::string>::iterator iterator;
		std::string dllPath, dllName;
		int i=0;
		// Get the root path here and initialize the deploymentConfigManager
		_dplyConfigMgr = new DeploymentConfigManager();
		_dplyConfigMgr->initialize(_webRoot.c_str());
		vDllNames = _dplyConfigMgr->getAllDeploymentDLLNames();

		for(iterator = vDllNames.begin(); iterator != vDllNames.end(); iterator++)
		{

#ifdef WIN32
	
			dllName = vDllNames.operator[](i);
			//MessageLogger::LogMessage('D', "ServletContainer::initialize", "DLL Name is %s", dllName.c_str());
			// Construct the complete path here using _webRoot and dllName
			dllPath.erase();
			dllPath.append(_webRoot);
			dllPath.append("../modules/");
			dllPath.append(dllName);
			//MessageLogger::LogMessage('D', "ServletContainer::initialize", "DLL Path is %s", dllPath.c_str());
			LoadLibrary(dllPath.c_str());

#else
			// IMPORTANT - Current code work for Unix in case of 
			// multi DLL model.

			// Need to call the appropriate function on unix
			// Just a dummy call to load the user_servlet_dll
			get_user_servlet_lib_factory();
#endif

			i++;
		}

		_initialized = true;
	}
	MessageLogger::LogMessage('t', "servletContainer", "End Servlet Container initialize");
}

