/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef SERVLET_CONTAINER_H
#define SERVLET_CONTAINER_H

// This class models a servlet container. It serves as an intermediary between
// the web server's module/extension and the servlet implementation and
// servlet itself.
// The webserver's modules will interface with only this component in the
// servlet implementation.

#include <string>
#include <vector>


#include "HttpServletRequest.h"
#include "HttpServletResponse.h"
#include "base_servlet/ServletException.h"
#include "DeploymentConfigManager.h"

#include "servlet_container_dll.h"

class SessionManager;

class SERVLET_CONTAINER_DLL_API ServletContainer
{

public:
	ServletContainer(std::string webRoot);
	
	virtual ~ServletContainer();



	// Will call the SessionManager to get an old/new session 
	// The module/extension will read the entire body 
	// and give it to this class. This class will call some parser class 
	// to do the parsing of 
	// query parameters and the body to populate the paramters list
	// HttpServletResponse object takes care of sending the response
	// back.
	// The web extension a custom (derived) ServletOutputStream object 
	// has to be implemented to support custom APIs for writing to 
	// the broswer. For e.g. in Apache's case ap_rprintf/ap_rputs etc. 
	int serviceRequest(HttpServletRequest & httpReq, HttpServletResponse & resp ) throw(ServletException);

	void setWebRoot(std::string rootPath)
	{
		_webRoot = rootPath;
	}

private:

	void initialize();

	bool _initialized;

	// handle to SessionManager 
	SessionManager *_sessionMgr;
	DeploymentConfigManager *_dplyConfigMgr;
	std::string _webRoot;

};



#endif