/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#pragma warning ( disable : 4786 )

#include "session_cache_mgr.h"
#include <iostream.h>

SessionCacheManager::SessionCacheManager()
{
	
}

//Returns the Session Object, NULL if not present
HttpSession* SessionCacheManager::getSessionObj(std::string sessionId)
{
	session_map_iterator iterator;
	
	ACE_Read_Guard<ACE_RW_Thread_Mutex> rg(_sessionMapMutex);

	iterator = _sessionMap.find(sessionId);
	
	if (iterator != _sessionMap.end())
	{
		return (*iterator).second;
	}
	else
	{
		return NULL;
	}
	
}

HttpSession* SessionCacheManager::getSessionObjUnProtected(std::string sessionId)
{
	session_map_iterator iterator;
	
	iterator = _sessionMap.find(sessionId);
	
	if (iterator != _sessionMap.end())
	{
		return (*iterator).second;
	}
	else
	{
		return NULL;
	}
	
}

//  0 on success 
int SessionCacheManager::addSessionObj(std::string sessionId, HttpSession * httpSessionPtr)
{

	// TO DO
	// Acquire the mutex here
	ACE_Write_Guard<ACE_RW_Thread_Mutex> wg(_sessionMapMutex);

	if (getSessionObjUnProtected(sessionId) != NULL)
	{
		return -1;
	}
	else
		_sessionMap[sessionId] = httpSessionPtr;

	return 0;
}

// 0 on success
int SessionCacheManager::destroySessionObj(std::string sessionId)
{

	HttpSession *sessionPtr;
	// TO DO
	// Acquire the mutex here
	ACE_Write_Guard<ACE_RW_Thread_Mutex> wg(_sessionMapMutex);

	sessionPtr = getSessionObjUnProtected(sessionId);
	if (sessionPtr != NULL)
	{
		// remove from the map
		_sessionMap.erase(sessionId);
		delete sessionPtr;

		return 0;
	}
	else
	{
		return 1;
	}

}
