/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef SESSION_CACHE_MGR_H
#define SESSION_CACHE_MGR_H

// This class is responsible for managing the session cache
// It is controlled / used by the SessionManager class

#include <string>
#include <map>
#include "ace\Synch.h"
#include "httpSession.h"

typedef std::map< std::string, HttpSession *> session_map;
typedef session_map::value_type  session_map_value_type;
typedef session_map::iterator session_map_iterator;

class SessionCacheManager
{
public:
	SessionCacheManager();


	virtual ~SessionCacheManager(){}

	// 0 on success
	int addSessionObj(std::string sessionId, HttpSession * httpSessionPtr);

	// 0 on success
	int destroySessionObj(std::string sessionId);

	//Returns the sessionObj, NULL if not present.
	HttpSession* getSessionObj(std::string sessionId);

private:

	// Returns the sessionObj, NULL if not present.
	// Used to modify the map object. Not locked using
	// mutex object.
	HttpSession* getSessionObjUnProtected(std::string sessionId);

	// This class should use a very efficient data structure
	// such that the search time for sessions is very scalable. 
	// Some thing like a hash table or a binary tree may be ideal 
	// to support a case of millions of sessions !

	// To start with we may go in for a simple object like std::map
	// Change data structures later

	session_map _sessionMap;
	ACE_RW_Thread_Mutex _sessionMapMutex;
};



#endif