/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#pragma warning ( disable : 4786 )

#include "session_cleanup_mgr.h"
#include "session_mgr.h"
#include <time.h>
#include <iostream.h>


SessionCleanupManager::SessionCleanupManager(SessionManager *sessionManager, SessionCacheManager *sessionCache)
{
	_sessionManager = sessionManager;
	_sessionCache = sessionCache;
	
}

SessionCleanupManager::~SessionCleanupManager()
{
	// No need to delete the objects _sessionManager and _sessionCache here
	// as they are not created by this object.
}



void SessionCleanupManager::registerSessionExpiry(std::string sessionId, long expTime)
{

	ACE_Guard<ACE_Thread_Mutex> g(_timeoutMapMutex);
	
	session_map_entry	mapEntry;
	mapEntry.first = expTime;
	mapEntry.second = sessionId;
	
	_sessionTimeoutMap.insert(mapEntry);
}

int SessionCleanupManager::updateSessionExpiry(std::string sessionId, long currentExpTime,
												long newExpTime)
{
	
	ACE_Guard<ACE_Thread_Mutex> g(_timeoutMapMutex);
	session_timeout_map_iterator iterator;
	
	iterator = _sessionTimeoutMap.find(currentExpTime);
	if (iterator == _sessionTimeoutMap.end())
		return -1;

	_sessionTimeoutMap.erase (iterator);
	registerSessionExpiry (sessionId, newExpTime);

	return 0;
}

void SessionCleanupManager::TimerCallBack(void *arg)
{
	SessionCleanupManager *sessionCleanupMgr = (SessionCleanupManager*)arg;
	sessionCleanupMgr->refreshSessionCache();
}

void SessionCleanupManager::refreshSessionCache()
{
	time_t currentTime;
	session_timeout_map_iterator timeoutIterator;
	session_timeout_map_iterator curIterator;
	std::string sessionId;
	ACE_Guard<ACE_Thread_Mutex> g(_timeoutMapMutex);
	ACE_OS::sleep (5);
	time (&currentTime);

	timeoutIterator = _sessionTimeoutMap.upper_bound(currentTime);
	
	for (curIterator = _sessionTimeoutMap.begin(); curIterator != timeoutIterator; curIterator++)
	{
		// Here get the sessionIds to be deleted and then remove them from the cache.
		sessionId = (*curIterator).second;
		_sessionManager->removeSessionObj (sessionId);
	}

	_sessionTimeoutMap.erase (_sessionTimeoutMap.begin(), timeoutIterator);


}