/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#pragma warning ( disable : 4786 )

#include "session_mgr.h"
#include <iostream.h>
#include <time.h>
#include "ace/Thread_Manager.h"

void * SessionManager::sessionCleanupFunction(void *arg)
{

	SessionCleanupManager *sessionCleanupMgr;
	ACE_Reactor reactor;
	int timerId;
	sessionCleanupMgr = (SessionCleanupManager*)arg;
	
	timerId = reactor.schedule_timer (sessionCleanupMgr, 0, ACE_Time_Value (5), ACE_Time_Value (10));

	while (ACE_Thread_Manager::instance()->testcancel (ACE_Thread::self()) == 0)
	{
		// Thread is active. 
		reactor.handle_events();
		
	}

	// Thread has been cancelled. Exit from here.

	return 0;
}

SessionManager::SessionManager()
{
	ACE::init();
	_sessionIdCount = 0;
	_sessionCache = new SessionCacheManager();
	_sessionCleanupMgr = new SessionCleanupManager(this, _sessionCache);
	initTimer();
}

SessionManager::~SessionManager()
{

	ACE_Thread_Manager *threadManager = ACE_Thread_Manager::instance();
	// threadManager->cancel_grp(1);

	delete _sessionCache;
	delete _sessionCleanupMgr;

}

void SessionManager::initTimer()
{

	ACE_Thread_Manager *threadManager = ACE_Thread_Manager::instance();
	
	if(threadManager->spawn_n (1, ACE_THR_FUNC (sessionCleanupFunction), 
												(void*)_sessionCleanupMgr, THR_NEW_LWP,
												ACE_DEFAULT_THREAD_PRIORITY,
												1) == -1)
	{
		// cout << "Failed to spawn new ACE Thread " << endl;
	}						

}

// generate session Id
std::string SessionManager::generateSessionId()
{
	int len = 0, lenTotal = 0;
	std::string sessionId("");
	time_t t1;
	char buffer[20];
	ACE_Guard<ACE_Thread_Mutex> g(_sessionCountMutex);
	
	time(&t1);
	_sessionIdCount++;
	_ultoa(_sessionIdCount,buffer,10);
	sessionId.append(buffer);
	_ultoa(t1,buffer,10);
	sessionId.append(buffer);

	return sessionId;

}

HttpSession * SessionManager::getSessionObj(std::string sessionId)
{
	HttpSession *sessionPtr;
	long currentExpiryTime;

	sessionPtr = _sessionCache->getSessionObj(sessionId);

	if (sessionPtr == NULL )
	{
		// Session expired.
		return NULL;
	}
	else
	{
		// update the currentExpTime here from the HttpSession object
		
		currentExpiryTime = sessionPtr->getLastAccessedTime() + sessionPtr->getMaxInactiveInterval();
		sessionPtr->setLastAccessesTime(time(NULL));

		if (_sessionCleanupMgr->updateSessionExpiry (sessionId, currentExpiryTime, sessionPtr->getLastAccessedTime() + sessionPtr->getMaxInactiveInterval()) != 0)
		{
			// Session expired
			return NULL;
		}
		return sessionPtr;
	}

}


int SessionManager::addSessionObj(HttpSession* httpSessionPtr)
{

	std::string sessionId;

	sessionId = httpSessionPtr->getId();
	if (_sessionCache->addSessionObj ( sessionId, httpSessionPtr) != 0)
	{
		return -1;
	}
	
	httpSessionPtr->setLastAccessesTime(time(NULL));

	_sessionCleanupMgr->registerSessionExpiry(sessionId, httpSessionPtr->getLastAccessedTime () + httpSessionPtr->getMaxInactiveInterval());
	return 0;

}

int SessionManager::removeSessionObj (std::string sessionId)
{

	if( _sessionCache->destroySessionObj(sessionId) == 0)
	{
		return 0;
	}

	return -1;
}

long SessionManager::getSessionCount()
{
	return _sessionIdCount;
}
