/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef SESSION_MGR_H
#define SESSION_MGR_H

// This class manages the creation and destruction of a user session
// It will have methods to generate a sessionId for the user.
// It will also create a UserSession Object and save it in the
// SessionCacheManager. 
// It will also register a sessionId with the SessionCleanupManager 
// telling it the Session expiry time. If the SessionManager does not
// re-register a session with the SessionCleanupManager before the
// expiry time, then the SessionCleanupManager will tell the SessionCacheManager
// to destroy the session.

#include "HttpSession.h"
#include "session_cache_mgr.h"
#include "session_cleanup_mgr.h"
#include "ace\Synch.h"

class SessionManager
{

public:

	SessionManager();

	virtual ~SessionManager();

	// returns 0 on success
	int addSessionObj(HttpSession* httpSessionPtr);

	// returns NULL on any error
	HttpSession * getSessionObj(std::string sessionId);

	// 0 on success
	int removeSessionObj (std::string sessionId);

	// Returns the total number of active sessions
	long getSessionCount();

	static void * sessionCleanupFunction(void *arg);
	
	// generate session Id
	std::string generateSessionId();
	
	// Returns the SessionCleanupManager instance
	SessionCleanupManager * getSessionCleanupManager()
	{ return _sessionCleanupMgr; }
private:


	// initialize the timer thread
	void initTimer();

	long					_sessionIdCount;
	SessionCacheManager		*_sessionCache;
	SessionCleanupManager	*_sessionCleanupMgr;
	ACE_Thread_Mutex		_sessionCountMutex;

};

#endif