/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#pragma warning ( disable : 4786 )

#include "SOAPHandlerFactory.h"
#include "MessageLogger.h"


SOAPHandlerFactory * SOAPHandlerFactory::_TheSOAPHandlerFactory = NULL;
bool SOAPHandlerFactory::_Initialized=false;

SOAPHandlerFactory* SOAPHandlerFactory::instance()
{
	if(_TheSOAPHandlerFactory== NULL)
		_TheSOAPHandlerFactory = new SOAPHandlerFactory();

	return _TheSOAPHandlerFactory;
}

void SOAPHandlerFactory::registerToFactory (const char* soapResName, struct SOAPHandlerStruct * soapHdlrStruct)
{
	MessageLogger::LogMessage('D', "SOAPHandlerFactory", "Inside SOAPHandlerFactory::registerToFactory(); soapResName = \"%s\".", soapResName);

	registeredSOAPHdlrStructMap::iterator regFnIter;
	regFnIter = instance()->_soapHdlrMap.find(soapResName);
	if(regFnIter ==  instance()->_soapHdlrMap.end() )
	{
		// If not present insert otherwise don't do anything
		instance()->_soapHdlrMap.insert(registeredSOAPHdlrStructMap::value_type(soapResName, soapHdlrStruct) );
	}
}

struct SOAPHandlerStruct * SOAPHandlerFactory::getSOAPHandlerStruct (const char* soapResourceName)
{
	
	//MessageLogger::LogMessage('D', "SOAPHandlerFactory", "Inside SOAPHandlerFactory::getServlet() \"%s\"", "");
	registeredSOAPHdlrStructMap::iterator regFnIter;
	regFnIter = instance()->_soapHdlrMap.find(soapResourceName);
	if(regFnIter !=  instance()->_soapHdlrMap.end() )
	{
		//MessageLogger::LogMessage('D', "SOAPHandlerFactory", "Found servlet \"%s\"", servletName);
		struct SOAPHandlerStruct * soapHdlrPtr = (*regFnIter).second;
		return soapHdlrPtr;
	}
	else
	{
		return NULL;
	}
	//MessageLogger::LogMessage('D', "SOAPHandlerFactory", "Leaving SOAPHandlerFactory::getServlet() \"%s\"", "");
}

