/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef __SOAP_HANDLER_FACTORY_H
#define __SOAP_HANDLER_FACTORY_H

#include "SOAP.h" // For the dll model to work

#include "stdsoap2.h"

// The factory class to create cpp user servlets.
#include <string>
#include <map>

typedef  void (*soap_init_fn)(struct soap *); 
typedef  int (*soap_serve_fn)(struct soap *);  
typedef  void (*soap_end_fn)(struct soap*); 

struct SOAPHandlerStruct
{
	soap_init_fn soap_init_fn_ptr;
    soap_serve_fn soap_serve_fn_ptr;
    soap_end_fn soap_end_fn_ptr;
};

// typedef  (SOAPHandlerStruct *) SOAPHandlerStructPtr;
//typedef  HttpServlet* (*registerFunction) (); 
typedef std::map< std::string, struct SOAPHandlerStruct * > registeredSOAPHdlrStructMap;

typedef registeredSOAPHdlrStructMap::value_type  registeredSOAPHdlrStructMapType;



//Define the MACRO's to be used by the servlets to register
//with the factory.

//#define QUOTE ""
// #define formServletName(A, QUOTE) QUOTE##A##QUOTE

// Have to introduce this second parameter as it is very difficult to 
// use a macro parameter value in a string. So expecting somebody 
// to call REGISTER_SERVLET(TryExampleServlet, "TryExampleServlet")

#define REGISTER_SOAP_RESOURCE(SOAP_RESOURCE_NAME_STR, SOAP_RESOURCE_NAME) \
	SOAPHandlerStruct SOAPHandlerStruct##SOAP_RESOURCE_NAME##Instance; \
\
struct dummySoap##SOAP_RESOURCE_NAME { \
	dummySoap##SOAP_RESOURCE_NAME() \
{\
	SOAPHandlerStruct##SOAP_RESOURCE_NAME##Instance.soap_init_fn_ptr = soap_init; \
	SOAPHandlerStruct##SOAP_RESOURCE_NAME##Instance.soap_serve_fn_ptr = soap_serve; \
	SOAPHandlerStruct##SOAP_RESOURCE_NAME##Instance.soap_end_fn_ptr = soap_end; \
	SOAPHandlerFactory::registerToFactory( SOAP_RESOURCE_NAME_STR, &SOAPHandlerStruct##SOAP_RESOURCE_NAME##Instance ); \
}\
};\
	struct dummySoap##SOAP_RESOURCE_NAME dummySoapInstance##SOAP_RESOURCE_NAME##;


// Exporting this class 
// TO DO put all such stuff under ifdef - make it portable

class SOAP_API SOAPHandlerFactory 
{
public:
	
	virtual ~SOAPHandlerFactory() {}
	
	static void registerToFactory (const char* soapResourceName, struct SOAPHandlerStruct * soapHdlrStruct);

	static struct SOAPHandlerStruct *getSOAPHandlerStruct (const char* soapResourceName);

	static SOAPHandlerFactory * instance();

	// The below function is used when user servlets lib is used a 
	// a dll. It will then overwrite the user_servlets_lib.dll's factory
	// with the one in this holder. 
	// This is only a temporary solution - for a permanent solution
	// the cpp_servlets_lib should be made a dll. Or atleast the 
	// SOAPHandlerFactory alone should be made into a separate dll.
	static setInstance(SOAPHandlerFactory * hdlrFactory)
	{
		//if(_TheSOAPHandlerFactory)
		//	delete _TheSOAPHandlerFactory;
		if(hdlrFactory != NULL)
			_TheSOAPHandlerFactory = hdlrFactory;
	}
	
	//Is the factory initialized with all the soap app dll 
	// registrations
	static bool Initialized()
	{ return _Initialized; }

	static void SetInitialized(bool val=true)
	{ _Initialized = val; }

private:

	 SOAPHandlerFactory() {}
	 registeredSOAPHdlrStructMap _soapHdlrMap;

	 // singleton instance
	 static SOAPHandlerFactory *_TheSOAPHandlerFactory;

	 static bool _Initialized;

};





#endif // __SOAP_HANDLER_FACTORY_H