/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#pragma warning ( disable : 4786 )

#include "gsoap_overwrite.h"

#include "ServletFactory.h"
#include "SOAPServlet.h"
#include "MessageLogger.h"
#include "SOAPHandlerFactory.h"


// This macro registers the servlet to the servlet factory
// Give the name of the servlet same as the servlet class name

// Commenting this out temparorily to make work with static lib
// Should retain this as the DLL model works.
// Uncommenting it back as now the dll model is working !!
REGISTER_SERVLET( SOAPServlet, "SOAPServlet")


#define SOAP_RESOURCE_KEY		"soap_res"
#define SOAP_APP_LOCATION		"soap_app_location"

void SOAPServlet::doGet(HttpServletRequest & req, HttpServletResponse & resp)
	throw( ServletException)
{	
	MessageLogger::LogMessage('D', "SOAPServlet", "Inside SOAPServlet::doGet()");

	ServletOutputStream & srvltOpStream = resp.getOutputStream();
	// Just says hello from its name and 
	// writes all the parameters back


	std::string soapRes = req.getParameter(SOAP_RESOURCE_KEY);
	std::string soapResDll;
	bool soapResFound = false;

	MessageLogger::LogMessage('D', "SOAPServlet", "Soap Resource name = %s", soapRes.c_str());

	if(!SOAPHandlerFactory::Initialized())
	{
		initialize();
	}

	
	// GET THE gsoap wrapper struct from the SOAPHandlerFactory

	struct SOAPHandlerStruct * hdlrStruct = SOAPHandlerFactory::getSOAPHandlerStruct(soapRes.c_str() );

	if(hdlrStruct == NULL)
	{
		// return error
		MessageLogger::LogMessage('E', "SOAPServlet", "Resource %s not registered to SOAPHandlerFactory.", soapRes.c_str() );
		resp.sendError(HttpServletResponse::SC_NOT_IMPLEMENTED); // 501 not implemented
		return;
	}

	//std::vector<std::string> attrList = req.getAttributeNames();

	resp.setContentType("text/xml");

	///** Temp comment out 

	struct soap psoap;
	gsoapUserData pRqConf;
	pRqConf._srvltReq = &req;
	pRqConf._srvltResp = &resp;

	
	// soap_init(&psoap);
	(*hdlrStruct->soap_init_fn_ptr) (&psoap);

	psoap.user = &pRqConf;
	set_callbacks(&pRqConf, &psoap);

	MessageLogger::LogMessage('D', "SOAPServlet", "Called set_callbacks()");
	
	// int nRet = soap_serve(&psoap); 
	int nRet = (*hdlrStruct->soap_serve_fn_ptr)(&psoap);

	MessageLogger::LogMessage('D', "SOAPServlet", "Finished soap_serve()");
	
	// soap_end(&psoap);
	(*hdlrStruct->soap_end_fn_ptr)(&psoap);

	// **/

	
	// This line is very important (atleast for now) as it won't write 
	// anything to client otherwise.
	resp.flushBuffer();

	
	MessageLogger::LogMessage('D', "SOAPServlet", "After calling flush()");
}

void SOAPServlet::doPost(HttpServletRequest & req, HttpServletResponse & resp)
	throw (ServletException)
{
	doGet(req, resp);
}

void SOAPServlet::initialize()
{
	MessageLogger::LogMessage('D', "SOAPServlet", "Inside initialize() method." );

	ServletConfig & srvltCfg = getServletConfig();
		
	std::string soapAppdir = srvltCfg.getInitParameter(SOAP_APP_LOCATION);

	MessageLogger::LogMessage('D', "SOAPServlet", "Soap app dir  = %s", soapAppdir.c_str());
	

	ServletContext & srvltCtxt = srvltCfg.getServletContext();

	std::vector<std::string> paramList = srvltCtxt.getInitParameterNames();
	std::string soapAppDll, soapAppDllFullPath;

	for(int i=0; i<paramList.size(); i++)
	{
		soapAppDll = srvltCtxt.getInitParameter(paramList[i]);
		MessageLogger::LogMessage('D', "SOAPServlet", "Soap app Name  = %s; Soap app Dll = %s", paramList[i].c_str(), soapAppDll.c_str() );
		// Load the soap app dll/shared library
	    

#ifdef WIN32
		soapAppDllFullPath = soapAppdir + soapAppDll;
		LoadLibrary(soapAppDllFullPath.c_str());
#else
			// ADD CODE for loading the shared library on *NIX es.

		MessageLogger::LogMessage('E', "SOAPServlet", "The functionality to load a shared library on UNIX not added" );
		return;
#endif

	}

	// Set the flag to initialized in the singleton factory
	SOAPHandlerFactory::SetInitialized();
}
