/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "bankAccountDB.h"
#include <fstream>
#include <cstring>

#define FILENAME "cpp_servlet\\soap\\apps\\bankDB.txt"

int BankAccountDB::saveToFile()
{
	std::ofstream fileStream;
	fileStream.open(FILENAME, std::ofstream::out);

	account_map_iterator iterator;
	AccountDetails *accountDetails;

	//hardcoding the size as 1000
	char buffer[1000];

	for (iterator = _accountMap.begin(); iterator != _accountMap.end(); iterator++)
	{
		accountDetails = (*iterator).second;
		memset(buffer, 0, 1000);
		sprintf (buffer, "%ld%c%s%c%lf\n", accountDetails->accountNumber, '#', accountDetails->accountName.c_str(), '#', accountDetails->accountBalance);
		fileStream.write(buffer, strlen(buffer));
	}

	fileStream.flush();
	fileStream.close();
	return 0;
}

int BankAccountDB::readFromFile()
{
	std::ifstream fileStream;
	fileStream.open(FILENAME, std::ifstream::in);
	if (fileStream.bad())
		return 0;


	account_map_iterator iterator;
	AccountDetails *accountDetails;
	
	for (iterator = _accountMap.begin(); iterator != _accountMap.end(); iterator++)
	{
		accountDetails = (*iterator).second;
		delete accountDetails;
	}
	_accountMap.clear();

	_accountCount = 0;
	
	
	char *firstDelim;
	char *secondDelim;
	int length;

	//hardcoding the size as 1000
	char buffer[1000];
	char accountNumber[200];
	char accountName[500];
	char accountBalance[300];

	while (fileStream.good())
	{
		
		fileStream.getline(buffer,1000);

		if (strlen(buffer) < 1)
			break;

		accountDetails = new AccountDetails();
		_accountCount++;
		
		//Extract accountNumber and Balance here.
		firstDelim = strchr(buffer, '#');
		length = firstDelim - buffer;
		memset (accountNumber, 0, 200);
		strncpy(accountNumber, buffer, length);

		firstDelim++;
		secondDelim = strchr(firstDelim, '#');
		length = secondDelim - firstDelim;
		memset (accountName, 0, 500);
		strncpy(accountName, firstDelim, length);

		secondDelim++;
		memset(accountBalance, 0, 300);
		strncpy(accountBalance, secondDelim, strlen(secondDelim));

		accountDetails->accountNumber = atol(accountNumber);
		accountDetails->accountName = accountName;
		accountDetails->accountBalance = atof(accountBalance); 
		_accountMap[accountDetails->accountNumber] = accountDetails;

	}

	fileStream.close();

	return 0;
}

long BankAccountDB::addAccount (AccountDetails *accoutDetailsPtr)
{
	_accountCount++;
	accoutDetailsPtr->accountNumber = _accountCount;
	_accountMap[_accountCount] = accoutDetailsPtr;
	return _accountCount;
}

AccountDetails* BankAccountDB::getAccount (long accountNumber)
{
	account_map_iterator iterator;
	
	iterator = _accountMap.find(accountNumber);
	
	if (iterator != _accountMap.end())
	{
		return (*iterator).second;
	}
	else
	{
		return NULL;
	}
}

