/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#pragma warning ( disable : 4786 )

#include "bankAccountDB.h"

#include "soapH.h"
#include "SOAPHandlerFactory.h"

REGISTER_SOAP_RESOURCE("bank", bank)

int bank__createAccount (struct soap * s, char *name, Account *account);
int bank__getBalance(struct soap * s, long accountNumber, Account *account);
int bank__deposit(struct soap * s, long accountNumber, float amount, Account *account);
int bank__withdraw(struct soap * s, long accountNumber, float amount, Account *account);


struct Namespace namespaces[] =
{
  {"SOAP-ENV", "http://schemas.xmlsoap.org/soap/envelope/"},
  {"SOAP-ENC", "http://schemas.xmlsoap.org/soap/encoding/"},
  {"xsi", "http://www.w3.org/2001/XMLSchema-instance", "http://www.w3.org/*/XMLSchema-instance"},
  {"xsd", "http://www.w3.org/2001/XMLSchema", "http://www.w3.org/*/XMLSchema"},
  {"bank", "http://tempuri"},
  {NULL, NULL}
};


BankAccountDB myBankDB;

int bank__createAccount (struct soap * s, char *name, float amount, Account *account)
{
	account->status = 0;

	if (myBankDB.readFromFile() != 0)
		return SOAP_FAULT;

	AccountDetails *accountDetails = new AccountDetails();
	accountDetails->accountName = name;
	accountDetails->accountBalance = amount;

	account->accountNumber = myBankDB.addAccount(accountDetails);

	if (myBankDB.saveToFile() != 0)
		return SOAP_FAULT;

	return SOAP_OK;
}

int bank__getBalance(struct soap * s, long accountNumber, Account *account)
{

	account->status = 0;
	if (myBankDB.readFromFile() != 0)
		return SOAP_FAULT;

	AccountDetails *accountDetails = myBankDB.getAccount(accountNumber);
	if (accountDetails == NULL)
	{
		account->status = ACCOUNT_DONT_EXIST;
		return SOAP_OK;
	}

	account->amount = accountDetails->accountBalance;

	return SOAP_OK;
}

int bank__deposit(struct soap * s, long accountNumber, float amount, Account *account)
{
	account->status = 0;
	if (myBankDB.readFromFile() != 0)
		return SOAP_FAULT;

	AccountDetails *accountDetails = myBankDB.getAccount(accountNumber);
	if (accountDetails == NULL)
	{
		account->status = ACCOUNT_DONT_EXIST;
		return SOAP_OK;
	}

	accountDetails->accountBalance = (float)accountDetails->accountBalance + (float) amount;

	if (myBankDB.saveToFile() != 0)
		return SOAP_FAULT;

	return SOAP_OK;
}

int bank__withdraw(struct soap * s, long accountNumber, float amount, Account *account)
{
	account->status = 0;
	if (myBankDB.readFromFile() != 0)
		return SOAP_FAULT;

	AccountDetails *accountDetails = myBankDB.getAccount(accountNumber);
	if (accountDetails == NULL)
	{
		account->status = ACCOUNT_DONT_EXIST;
		return SOAP_OK;
	}

	if (accountDetails->accountBalance < amount)
	{
		account->status = NO_SUFFICIENT_BALANCE;
		return SOAP_OK;
	}

	accountDetails->accountBalance = (float)accountDetails->accountBalance - (float) amount;

	if (myBankDB.saveToFile() != 0)
		return SOAP_FAULT;

	return SOAP_OK;
}
