/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#pragma warning ( disable : 4786 )

#include "soapH.h"	// include generated proxy and SOAP support
#include <iostream.h>

#define INIT_BALANCE_LESS 100
#define NO_SUFFICIENT_BALANCE 101
#define ACCOUNT_DONT_EXIST 102


int main(int argc, char **argv)
{ struct soap soap;	// allocate gSOAP runtime environment
  char *host;
  char *port;

  char queryStr[200];
  strcpy(queryStr, "http://");
  if(argc >=3 )
  {
	  host = argv[1];
	  port = argv[2];
  }
  else 
  {
	  host = "localhost";
	  port = "80";
	  if (argc > 1)
	  {
		printf("Usage : \n");
		printf("1) %s <hostname> <portno> \n" , argv[0]);
		printf("2) %s \n   {Defaults used host=localhost; port=80} \n", argv[0]);
		exit(0);
	  }
	  
  }
  printf("Connecting to Host : %s ; port : %s\n", host, port);

  strcat(queryStr, host);
  strcat(queryStr, ":");
  strcat(queryStr, port);
  strcat(queryStr, "/cpp_servlet/soap/SOAPServlet?soap_res=bank");
  printf("URL = %s\n", queryStr);

  soap_init(&soap);	// must initialize

  float amount;
  long acNumber;

  char option='a';
  char acName[500];
  int retValue;

  while (option !='0')
  {
	  printf("\n\t\tWEB SERVICES EXAMPLE - BANK\n");

	  printf ("\n1 : \t create account\n");
	  printf ("2 : \t deposit\n");
	  printf ("3 : \t withdraw\n");
	  printf ("4 : \t check balance\n");
	  printf ("5 : \t exit");

	  fflush(stdin);
	  printf ("\n\nEnter the option here :");
	  scanf("%c", &option);

	  Account account;

	  switch (option)
	  {
	  case '1':
		  printf ("\nEnter the name:");
		  cin >> acName;

		  printf ("Enter the initial balance $:");
		  scanf ("%f", &amount);
		
		  //minimum of $100 required to start an account
		  if(amount < 100)
		  {
			printf ("\n\n\n**********************************************************\nA minimum balance of $100 is required to create an account\n**********************************************************\n");
			break;
		  }
		
		  retValue = soap_call_bank__createAccount(&soap, queryStr, NULL, acName, amount, &account);
		  if (retValue == 0)
			printf("\n\n\n**********************************************************\nAccount created successully. The accountNumber is %d\n**********************************************************\n", account.accountNumber);
		  else
		  { soap_print_fault(&soap, stderr);
			soap_print_fault_location(&soap, stderr);
		  }


		  break;
	  case '2':
		  printf ("\nEnter the account number:");
		  cin >> acNumber;

		  printf ("Enter the amount to be deposited $:");
		  scanf ("%f", &amount);
		  printf ("Amount to be deposited = %f\n", amount);

		  if (soap_call_bank__deposit(&soap, queryStr, NULL, acNumber, amount, &account) == 0){
			if (account.status == ACCOUNT_DONT_EXIST) 
			{
		   		printf ("\n\n\n**********************\nInvalid account number\n**********************\n");
				break;
			}

			if (soap_call_bank__getBalance(&soap, queryStr, NULL, acNumber, &account) == 0){
			  printf("\n\n\n*************************************************************\nAmount deposited successully. The current balance is %lf\n*************************************************************\n", account.amount);
			}
			else
			{  soap_print_fault(&soap, stderr);
			   soap_print_fault_location(&soap, stderr);
			}

		  }
		  else
		  { soap_print_fault(&soap, stderr);
			soap_print_fault_location(&soap, stderr);
		  }

		  break;
	  case '3':
		  printf ("\nEnter the account number:");
		  cin >> acNumber;

		  printf ("Enter the amount to be withdrawn $:");
		  scanf ("%f", &amount);
		  printf ("Amount to be withdrawn = %f\n", amount);
		
		  retValue = soap_call_bank__withdraw(&soap, queryStr, NULL, acNumber, amount, &account);
		  if (retValue == 0){

			if (account.status == ACCOUNT_DONT_EXIST) 
			{
				printf ("\n\n\n**********************\nInvalid account number\n**********************\n");
				break;
			}
			else if (account.status == NO_SUFFICIENT_BALANCE) 
			{
				printf ("\n\n\n*********************************\nNo sufficient balance to withdraw\n*********************************\n");
				break;
			}

			if (soap_call_bank__getBalance(&soap, queryStr, NULL, acNumber, &account) == 0){
			  printf("\n\n\n**************************************************************\nAmount withdrawn successully. The current balance is %lf\n**************************************************************\n", account.amount);
			}
			else
			{  soap_print_fault(&soap, stderr);
			   soap_print_fault_location(&soap, stderr);
			}

		  }
		  else
		  { soap_print_fault(&soap, stderr);
			soap_print_fault_location(&soap, stderr);
		  }

	  
		  break;
	  case '4':
		  printf ("\nEnter the account number:");
		  cin >> acNumber;

		  if (soap_call_bank__getBalance(&soap, queryStr, NULL, acNumber, &account) == 0){
		    if (account.status == ACCOUNT_DONT_EXIST) 
			{
				printf ("\n\n\n**********************\nInvalid account number\n**********************\n");
				break;
			}
			else
		        printf("\n\n\n***********************************\nThe current balance is $ %lf\n***********************************\n", account.amount);
		  }
		  else
		  {  soap_print_fault(&soap, stderr);
		   soap_print_fault_location(&soap, stderr);
		  }

		  break;
	  case '5':
		  exit (0);
	  default:

		  printf ("\n\n\n***************************************\nInvalid option. Select any valid option\n***************************************\n");
		  option='a';
		  break;
	  
	  }

  }




  return 0;
}

// The namespace mapping table is required and associates namespace prefixes with namespace names:

struct Namespace namespaces[] =
{
  {"SOAP-ENV", "http://schemas.xmlsoap.org/soap/envelope/"},
  {"SOAP-ENC", "http://schemas.xmlsoap.org/soap/encoding/"},
  {"xsi", "http://www.w3.org/2001/XMLSchema-instance", "http://www.w3.org/*/XMLSchema-instance"},
  {"xsd", "http://www.w3.org/2001/XMLSchema", "http://www.w3.org/*/XMLSchema"},
  {"bank", "http://tempuri"},
  {NULL, NULL}
};

