/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#pragma warning ( disable : 4786 )

#include "tryExampleServlet.h"
#include "ServletFactory.h"


// This macro registers the servlet to the servlet factory
// Give the name of the servlet same as the servlet class name

// Commenting this out temparorily to make work with static lib
// Should retain this as the DLL model works.
// Uncommenting it back as now the dll model is working !!
REGISTER_SERVLET( TryExampleServlet, "TryExampleServlet")


void TryExampleServlet::doGet(HttpServletRequest & req, HttpServletResponse & resp)
	throw( ServletException)
{
	
	ServletOutputStream & srvltOpStream = resp.getOutputStream();
	// Just says hello from its name and 
	// writes all the parameters back
	std::vector<std::string> attrList = req.getAttributeNames();

	resp.setContentType("text/html");

	
	

		// Get the session object

	HttpSession & sessionObj = req.getSession(true);

	Cookie ckie("TryExampleServlet_cookie1", "try_cookie_val1");
	ckie.setComment("Just_a_trial_cookie");

	resp.addCookie(ckie);

	srvltOpStream.print("Hello from TryExampleServlet<BR> \n");

	
	srvltOpStream.print("The parameters sent from the client are listed below:<BR> \n");
	
	for(int i=0; i<attrList.size() ;i++)
	{
		std::string strParamValue;
		strParamValue = attrList[i];
		strParamValue += ": ";
		strParamValue += req.getParameter(attrList[i]);
		strParamValue += " <BR>\n";
		srvltOpStream.print(strParamValue);
	}
	
	std::vector<Cookie> & clientCookieList = req.getCookies();
	
	srvltOpStream.print("The cookies sent from the client are listed below:<BR> \n");

	for(i=0; i<clientCookieList.size() ;i++)
	{
		Cookie & ckie = clientCookieList[i];
		std::string strParamValue;
		strParamValue = ckie.getName();
		strParamValue += " = ";
		strParamValue += ckie.getValue();
		strParamValue += " <BR>\n";
		srvltOpStream.print(strParamValue);
	}



	srvltOpStream.print("Session Id : ");
	srvltOpStream.print(sessionObj.getId().c_str() );
	srvltOpStream.print(" <br>\n");

	if(req.newSessionCreated())
		sessionObj.setAttribute("session_key", "cpp_servlet_val");
	else
	{
		srvltOpStream.print("key : ");
		srvltOpStream.print(sessionObj.getAttribute("session_key").c_str() );
		srvltOpStream.print("<br> \n");
	}


	
	// This line is very important (atleast for now) as it won't write 
	// anything to client otherwise.
	resp.flushBuffer();
	
}

void TryExampleServlet::doPost(HttpServletRequest & req, HttpServletResponse & resp)
	throw (ServletException)
{
	doGet(req, resp);
}
